// SPODS--Simple Physics Object based Detector Simulation
// Copyright (C) 2011, 2012, 2013 Justin Hugon

#include "PhysicsObjectBuilder.h"
#include "PhysicsObjectFunctions.h"
#include <vector>
#include "TLorentzVector.h"
#include <csignal>
#include "TParticlePDG.h"

namespace SPODS
{

PhysicsObjectBuilder::PhysicsObjectBuilder(YamlConfig* config):
	fGenParticles(0),fNGenParticles(0),tRandom()
{
	fConfig = config;
	databasePDG = new TDatabasePDG();

  	// choose a jet definition
	if(fConfig->jetAlg == "antikt")
  	  jet_def= new fastjet::JetDefinition(fastjet::antikt_algorithm, fConfig->jetAlgR);
	else if(fConfig->jetAlg == "kt")
  	  jet_def= new fastjet::JetDefinition(fastjet::kt_algorithm, fConfig->jetAlgR);
	else if(fConfig->jetAlg == "ca")
  	  jet_def= new fastjet::JetDefinition(fastjet::cambridge_algorithm, fConfig->jetAlgR);
	else
	{
	  std::cout << "Error: jetAlg isn't antikt, kt, or ca!" << std::endl;
	  throw;
	}

}

void
PhysicsObjectBuilder::LoadGenParticles(TClonesArray* genParticles)
{
	fGenParticles = genParticles;
	fNGenParticles = fGenParticles->GetEntries();
}

void
PhysicsObjectBuilder::MakeElectrons(TClonesArray* electrons)
{
	//std::cout << "Cuts Electron pt: " << fConfig->ptElectronMin << 
	//	" eta: " << fConfig->etaElectronMax << std::endl;
	electrons->Clear();
	int outSize = electrons->GetSize();
	int iOut = 0;
	for(int iGen = 0; iGen < fNGenParticles; iGen++)
	{
	  TParticle* genPart = (TParticle*) (fGenParticles->At(iGen));
	  if(genPart->GetStatusCode()==1 && abs(genPart->GetPdgCode()) == 11)
	  {
	    float absEta = fabs(genPart->Eta());
	    if (absEta >= fConfig->etaElectronMax)
		break;
	    float pt = genPart->Pt();
	    float sigma=0.0;
	    if (pt < fConfig->electronResPtCut)
	    	sigma = fConfig->electronResConstantTerm;
	    else
	    {
	      unsigned iEtaRange = 0;
	      bool etaRangeFound = false;
	      for(std::vector<std::pair<float,float> >::const_iterator etaRange =
		fConfig->photonResEtaRanges.begin(); 
		etaRange != fConfig->photonResEtaRanges.end(); etaRange++)
	      {
		if(absEta > etaRange->first && absEta < etaRange->second)
		{
		  etaRangeFound = true;
		  break;
		}
		iEtaRange++;
	      }
	      if (!etaRangeFound)
	      {
		std::cout << "Error: Electron inside etaElectronMax, " <<
			"but not within a photonResEtaRange! Setting Sigma=0.0" << std::endl;
	      }
	      else
	      {
	        sigma = sqrt(pow(fConfig->photonResConstTerm.at(iEtaRange),2) +
			pow(fConfig->photonResOneOverETerm.at(iEtaRange)/pt,2) +
			pow(fConfig->photonResOneOverSqrtETerm.at(iEtaRange),2)/pt
		  );
	      }
	    }
	    float scaleFactor = tRandom.Gaus(1.0,sigma);
	    TLorentzVector tmpP;
	    genPart->Momentum(tmpP);
	    tmpP *= scaleFactor;
	    if(pt > 1.0)
	    {
		TLorentzVector tmpV;
		genPart->ProductionVertex(tmpV);
		TParticle tmpParticle(genPart->GetPdgCode(),1,-1,-1,-1,-1,
				tmpP,tmpV);
		setGenScalingFactor(&tmpParticle,1.0/scaleFactor);
		new(electrons->operator[](iOut)) TParticle(tmpParticle);
		iOut++;
		if(iOut>=outSize)
			break;

		if (fabs(genPart->Pt()-tmpP.Pt()/scaleFactor) >0.0001)
			std::cout << "Error: el genPt: " <<genPart->Pt()<< ", recoPt/SF:" << tmpP.Pt()/scaleFactor<< std::endl;
	    }
	  }
	}
}

void
PhysicsObjectBuilder::MakeMuons(TClonesArray* muons)
{
	//std::cout << "Cuts muon pt: " << fConfig->ptMuonMin << 
	//	" eta: " << fConfig->etaMuonMax << std::endl;
	muons->Clear();
	int outSize = muons->GetSize();
	int iOut = 0;
	for(int iGen = 0; iGen < fNGenParticles; iGen++)
	{
	  TParticle* genPart = (TParticle*) (fGenParticles->At(iGen));
	  if(genPart->GetStatusCode()==1 && abs(genPart->GetPdgCode()) == 13)
	  {
	    float absEta = fabs(genPart->Eta());
	    if (absEta >= fConfig->etaMuonMax)
		break;
	    float pt = genPart->Pt();
	    unsigned iEtaRange = 0;
	    bool etaRangeFound = false;
	    for(std::vector<std::pair<float,float> >::const_iterator etaRange =
		fConfig->muonResEtaRanges.begin(); 
		etaRange != fConfig->muonResEtaRanges.end(); etaRange++)
	    {
		if(absEta > etaRange->first && absEta < etaRange->second)
		{
		  etaRangeFound = true;
		  break;
		}
		iEtaRange++;
	    }
	    float sigma = 0.0;
	    if (!etaRangeFound)
	    {
		std::cout << "Error: muon inside etaMuonMax, " <<
			"but not within a muonResEtaRange! Setting Sigma=0.0" << std::endl;
	    }
	    else
	    {
	      if(pt>fConfig->muonResHighPtCut.at(iEtaRange))
	      {
		sigma = fConfig->muonResHighPtConstTerm.at(iEtaRange) +
			fConfig->muonResHighPtSqrtTerm.at(iEtaRange)*sqrt(pt/1000.0);
	      }
	      else
	      {
		sigma = fConfig->muonResConstTerm.at(iEtaRange) +
			fConfig->muonResSqrtTerm.at(iEtaRange)*sqrt(pt/1000.0);
	      }
	    }
	    float scaleFactor = tRandom.Gaus(1.0,sigma);
	    TLorentzVector tmpP;
	    genPart->Momentum(tmpP);
	    tmpP *= scaleFactor;
	    if(tmpP.Pt() > 2.0)
	    {
		TLorentzVector tmpV;
		genPart->ProductionVertex(tmpV);

		TParticle tmpParticle(genPart->GetPdgCode(),1,-1,-1,-1,-1,
				tmpP,tmpV);
		setGenScalingFactor(&tmpParticle,1.0/scaleFactor);
		new(muons->operator[](iOut)) TParticle(tmpParticle);
		iOut++;
		if(iOut>=outSize)
			break;

		if (fabs(genPart->Pt()-tmpP.Pt()/scaleFactor) >0.0001)
			std::cout << "Error: e genPt: " <<genPart->Pt()<< ", recoPt/SF:" << tmpP.Pt()/scaleFactor<< std::endl;
	    }
	  }
	}
}

void
PhysicsObjectBuilder::MakePhotons(TClonesArray* photons)
{
	//std::cout << "Cuts Photons pt: " << fConfig->ptPhotonMin << 
	//	" eta: " << fConfig->etaPhotonMax << std::endl;
	photons->Clear();
	int outSize = photons->GetSize();
	int iOut = 0;
	for(int iGen = 0; iGen < fNGenParticles; iGen++)
	{
	  TParticle* genPart = (TParticle*) (fGenParticles->At(iGen));
	  if(genPart->GetStatusCode()==1 && abs(genPart->GetPdgCode()) == 22)
	  {
	    float absEta = fabs(genPart->Eta());
	    if (absEta >= fConfig->etaPhotonMax)
		break;
	    float pt = genPart->Pt();
	    unsigned iEtaRange = 0;
	    bool etaRangeFound = false;
	    for(std::vector<std::pair<float,float> >::const_iterator etaRange =
		fConfig->photonResEtaRanges.begin(); 
		etaRange != fConfig->photonResEtaRanges.end(); etaRange++)
	    {
		if(absEta > etaRange->first && absEta < etaRange->second)
		{
		  etaRangeFound = true;
		  break;
		}
		iEtaRange++;
	    }
	    float sigma = 0.0;
	    if (!etaRangeFound)
	    {
		std::cout << "Error: photon inside etaPhotonMax, " <<
			"but not within a photonResEtaRange! Setting Sigma=0.0" << std::endl;
	    }
	    else
	    {
	      sigma = sqrt(pow(fConfig->photonResConstTerm.at(iEtaRange),2) +
			pow(fConfig->photonResOneOverETerm.at(iEtaRange)/pt,2) +
			pow(fConfig->photonResOneOverSqrtETerm.at(iEtaRange),2)/pt
		  );
	    }
	    //float sigmaCheck = sqrt(pow(0.036/sqrt(pt),2)+pow(0.185/pt,2)+pow(0.0066,2));
	    //std::cout << "photon Pt: " << pt << "     sigma: " << sigma << 
	    //			"     sigma check: "<< sigmaCheck << "\n";
	    float scaleFactor = tRandom.Gaus(1.0,sigma);
	    TLorentzVector tmpP;
	    genPart->Momentum(tmpP);
	    tmpP *= scaleFactor;
	    if(pt > 1.0)
	    {
		TLorentzVector tmpV;
		genPart->ProductionVertex(tmpV);
		TParticle tmpParticle(genPart->GetPdgCode(),1,-1,-1,-1,-1,
				tmpP,tmpV);
		setGenScalingFactor(&tmpParticle,1.0/scaleFactor);
		new(photons->operator[](iOut)) TParticle(tmpParticle);
		iOut++;
		if(iOut>=outSize)
			break;

		if (fabs(genPart->Pt()-tmpP.Pt()/scaleFactor) >0.0001)
			std::cout << "Error: gam genPt: " <<genPart->Pt()<< ", recoPt/SF:" << tmpP.Pt()/scaleFactor<< std::endl;
	    }
	  }
	}
}

void
PhysicsObjectBuilder::MakeJets(TClonesArray* jets)
{
	//std::cout << "Cuts Jets pt: " << fConfig->ptJetMin << 
	//	" eta: " << fConfig->etaJetMax << std::endl;
	TClonesArray* tempJetGenParticles = 
		new TClonesArray("TParticle",fNGenParticles);
	int iTemp=0;
	for(int iGen = 0; iGen < fNGenParticles; iGen++)
	{
	  TParticle* genPart = (TParticle*) (fGenParticles->At(iGen));
	  if(genPart->GetStatusCode()==1)
	  {
	    int pdg = abs(genPart->GetPdgCode());
	    bool neutrino = (pdg == 12) || (pdg == 14) || (pdg == 16);
        bool bsm = pdg > 1000000;
	    bool clusterable = genPart->Pt() > 0.5 
			&& fabs(genPart->Eta()) < fConfig->etaJetMax
		&& !neutrino && pdg != 13 && !bsm;
	    if(clusterable)
	    {
		new(tempJetGenParticles->operator[](iTemp)) TParticle(*genPart);
		iTemp++;
	    }
	  }
	}
	GetJets(tempJetGenParticles, jets);
	delete tempJetGenParticles;
}

void 
PhysicsObjectBuilder::MakeMet(TClonesArray* physicsObjects, 
	double& met, double& phi, double& sumPt,
	double& trueMet, double& truePhi, double& trueSumPt
  	)
{
	TLorentzVector totP(0.0,0.0,0.0,0.0);
	trueSumPt=0.0;

	for(int iGen = 0; iGen < fNGenParticles; iGen++)
	{
	  TParticle* genPart = (TParticle*) (fGenParticles->At(iGen));
	  if(genPart->GetStatusCode()==1)
	  {
	    int pdg = abs(genPart->GetPdgCode());
	    bool neutrino = (pdg == 12) || (pdg == 14) || (pdg == 16);
        bool bsm = pdg > 1000000;
	    bool visibleParticle = genPart->Pt() > 0.5
			&& fabs(genPart->Eta()) < fConfig->etaJetMax
		&& !neutrino && !bsm;
	    if(visibleParticle)
	    {
		TLorentzVector tempP;
		genPart->Momentum(tempP);
		totP += tempP;
		trueSumPt += tempP.Pt();
	    }
	  }
	}
	trueMet = totP.Pt();
	truePhi = totP.Phi();

	TLorentzVector metVec;
	sumPt=0.0;
	unsigned nPO = physicsObjects->GetEntries();
	for(unsigned iPO = 0; iPO < nPO; iPO++)
	{
	  TParticle* tmpPart = (TParticle*) physicsObjects->At(iPO);
	  sumPt += tmpPart->Pt();

	  TLorentzVector tempP;
	  tmpPart->Momentum(tempP);
	  metVec += tempP;
	} 
	met = metVec.Pt();
	phi = metVec.Phi();
}

void 
PhysicsObjectBuilder::GetJets(TClonesArray* genParticles, TClonesArray* outJets)
{
  	std::vector<fastjet::PseudoJet> particles;
	int nGen = genParticles->GetEntries();
	TLorentzVector p(0.0,0.0,0.0,0.0);
	for(int iGen = 0; iGen < nGen; iGen++)
	{
	  TParticle* genPart = (TParticle*) (genParticles->At(iGen));
	  genPart->Momentum(p);
	  fastjet::PseudoJet tmpJet(p.Px(),p.Py(),p.Pz(),p.E());
	  tmpJet.set_user_index(genPart->GetPdgCode());
  	  particles.push_back(tmpJet); 
	}

  	// run the clustering, extract the jets
  	fastjet::ClusterSequence cs(particles, *jet_def);
  	std::vector<fastjet::PseudoJet> jets = 
		fastjet::sorted_by_pt(cs.inclusive_jets(fConfig->ptJetMin));
  	std::vector<fastjet::PseudoJet>::const_iterator jet;

	//Output 
	outJets->Clear();
	int outSize = outJets->GetSize();
	int iOut = 0;

	TLorentzVector tmpV;
  	for(jet = jets.begin();jet !=jets.end();jet++)
	{
	    int nConstituents = 0;
	    int nConstituentsCharged = 0;
	    int nConstituentsHad = 0;
	    GetNConstituents(*jet,nConstituents,nConstituentsCharged,
								nConstituentsHad);
	    if (nConstituents < fConfig->jetVetoNPartMin || 
		nConstituentsCharged < fConfig->jetVetoNChargMin ||
		nConstituentsHad < fConfig->jetVetoNHadMin)
		continue;

	    //Now on to normal smearing and kinetic vars part
	    TLorentzVector tmpP(jet->px(),jet->py(),jet->pz(),jet->e());
	    TLorentzVector genPart(tmpP);

	    float pt = tmpP.Pt();
	    float absEta = tmpP.Eta();
	    unsigned iEtaRange = 0;
	    for(std::vector<std::pair<float,float> >::const_iterator etaRange =
		fConfig->jetResEtaRanges.begin(); 
		etaRange != fConfig->jetResEtaRanges.end(); etaRange++)
	    {
		if(absEta > etaRange->first && absEta < etaRange->second)
		{
		  break;
		}
		iEtaRange++;
	    }
	    if (iEtaRange >= fConfig->jetResEtaRanges.size())
		iEtaRange = fConfig->jetResEtaRanges.size()-1;

	    float sigma = 0.0;
	    float scaleFactor = 1.0;
	    if (pt >= fConfig->jetResPtCut.at(iEtaRange))
	    {
	    	sigma = sqrt(pow(fConfig->jetResConstTerm.at(iEtaRange),2) +
			pow(fConfig->jetResOneOverETerm.at(iEtaRange)/pt,2) +
			pow(fConfig->jetResOneOverSqrtETerm.at(iEtaRange),2)/pt
			);
	    	scaleFactor = tRandom.Gaus(1.0,sigma);
	    	tmpP *= scaleFactor;
	    }
	    if(pt > 5.0)
	    {
		TParticle tmpParticle(-1,-1,-1,-1,-1,-1, tmpP, tmpV);
		setGenScalingFactor(&tmpParticle,1.0/scaleFactor);
		new(outJets->operator[](iOut)) TParticle(tmpParticle);
		iOut++;
		if(iOut>=outSize)
			break;

		if (fabs(genPart.Pt() - tmpP.Pt() / scaleFactor) > 0.001)
			std::cout << "Error: jet genPt: " <<genPart.Pt()<< ", recoPt/SF:" << tmpP.Pt()/scaleFactor<< ", SF: " << scaleFactor << ", sigma: " << sigma<< std::endl;
	    }
	}
}

void
PhysicsObjectBuilder::MakeIso(TClonesArray* physicsObjects, std::string type, float coneSize, float minPt, std::string particle)
{
   if(particle != "photon" && particle != "electron" && particle != "muon")
   {
	std::cout << "Error: PhysicsObjectBuilder::MakeIso() particle is "<< particle << ", should be photon, electron, or muon.  Exiting.\n";
	throw;
   }
	
   int nPhsysicsObjects = physicsObjects->GetEntries();
   for(int iPO=0; iPO < nPhsysicsObjects; iPO++)
   {
	TParticle* physObj = (TParticle*) (physicsObjects->At(iPO));
	float etaPO = physObj->Eta();
	float phiPO = physObj->Phi();
	float ptPO = physObj->Pt()*getGenScalingFactor(physObj);
	float iso = 0.0;
	for(int iGen = 0; iGen < fNGenParticles; iGen++)
	{
	  TParticle* genPart = (TParticle*) (fGenParticles->At(iGen));
	  if(genPart->GetStatusCode()==1)
	  {
	    if(genPart->Pt() < minPt)
		continue;
	    float dR = sqrt(pow(genPart->Eta()-etaPO,2) 
		+ pow(genPart->Phi()-phiPO,2));
	    if(dR > coneSize)
		continue;
	    int pdg = abs(genPart->GetPdgCode());
        bool bsm = pdg > 1000000;
	    if(bsm)
		continue;
	    bool neutrino = (pdg == 12) || (pdg == 14) || (pdg == 16);
	    bool caloParticle = !neutrino && pdg != 13;
	    bool charged = GetChargeFromPDG(pdg) != 0.0;
	    
	    if(type == "trk" && charged)
	    {
		iso += genPart->Pt();
	    }
	    else if(type == "calo" && caloParticle)
	    {
		iso += genPart->Pt();
	    }
	    else if(type == "combined" && (charged || caloParticle))
	    {
		iso += genPart->Pt();
	    }
	  }
	}
	if(
		((type == "calo") && (particle == "muon")) 
		|| 
		((type == "trk") && (particle == "photon")) 
	  )
	{
	  setIso(physObj,iso);
	}
	else
	{
	  setIso(physObj,iso-ptPO);
	}
   }
}

void 
PhysicsObjectBuilder::BTagJets(TClonesArray *jets, TClonesArray* allGenParticles)
{

   std::vector<TParticle> genBs;
   int nAllGenParticles = allGenParticles->GetEntries();
   for(int iGen=0; iGen < nAllGenParticles; iGen++)
   {
	TParticle* genPart = (TParticle*) (allGenParticles->At(iGen));
    unsigned pdg = abs(genPart->GetPdgCode());
    if (pdg > 1000000)
      continue;
	if(pdg != 5)
    {
      if(pdg < 100)
        continue;
      unsigned pdg2Digit = (pdg/10) % 10;
      unsigned pdg3Digit = (pdg/100) % 10;
      unsigned pdg4Digit = (pdg/1000) % 10;
      if(pdg2Digit != 5 && pdg3Digit != 5 && pdg4Digit != 5)
    	  continue;
    }
	genBs.push_back(*genPart);
	
   }

   int nJets = jets->GetEntries();
   for(int iJet=0; iJet < nJets; iJet++)
   {
	TParticle* jet = (TParticle*) (jets->At(iJet));
	float etaJet = jet->Eta();
	float phiJet = jet->Phi();
	float maxR = fConfig->jetAlgR;
	bool trueB = false;
	for (std::vector<TParticle>::const_iterator genB = genBs.begin();
		genB != genBs.end(); genB++)
	{
	    float dR = sqrt(pow(genB->Eta()-etaJet,2) 
		+ pow(genB->Phi()-phiJet,2));
	    if(dR < maxR)
	    {
		trueB = true;
		break;
	    }
	}
	bool bTag = false;
	if(trueB && tRandom.Uniform() <= fConfig->bTagEff)
	    bTag = true; 
	else if(!trueB && tRandom.Uniform() <= fConfig->bTagMistagRate)
	    bTag = true;
	setGenB(jet,trueB);
	setBTag(jet,bTag);
   }
}

void 
PhysicsObjectBuilder::CombinePhysicsObjects(TClonesArray* electrons, TClonesArray* muons, 
	TClonesArray* photons, TClonesArray* jets, TClonesArray* combined)
{
   int iOut = 0;
   int outSize = combined->GetSize();

   int nElectrons = electrons->GetEntries();
   for(int iElectron=0; iElectron < nElectrons; iElectron++)
   {
	TParticle* tmpParticle = (TParticle*) (electrons->At(iElectron));
	new(combined->operator[](iOut)) TParticle(*tmpParticle);
	iOut++;
	if(iOut>=outSize)
        {
		std::cout << "Error: N Physics Objects > combined objects TClonesArray Size!!" << std::endl;
		std::cout << "\t Not all objects included in MET calculation!" << std::endl;
		return;
        }
   }

   int nMuons = muons->GetEntries();
   for(int iMuon=0; iMuon < nMuons; iMuon++)
   {
	TParticle* tmpParticle = (TParticle*) (muons->At(iMuon));
	new(combined->operator[](iOut)) TParticle(*tmpParticle);
	iOut++;
	if(iOut>=outSize)
        {
		std::cout << "Error: N Physics Objects > combined objects TClonesArray Size!!" << std::endl;
		std::cout << "\t Not all objects included in MET calculation!" << std::endl;
		return;
        }
   }

   int nPhotons = photons->GetEntries();
   for(int iPhoton=0; iPhoton < nPhotons; iPhoton++)
   {
	TParticle* tmpParticle = (TParticle*) (photons->At(iPhoton));
	new(combined->operator[](iOut)) TParticle(*tmpParticle);
	iOut++;
	if(iOut>=outSize)
        {
		std::cout << "Error: N Physics Objects > combined objects TClonesArray Size!!" << std::endl;
		std::cout << "\t Not all objects included in MET calculation!" << std::endl;
		return;
        }
   }

   int nJets = jets->GetEntries();
   for(int iJet=0; iJet < nJets; iJet++)
   {
	TParticle* tmpParticle = (TParticle*) (jets->At(iJet));
	new(combined->operator[](iOut)) TParticle(*tmpParticle);
	iOut++;
	if(iOut>=outSize)
        {
		std::cout << "Error: N Physics Objects > combined objects TClonesArray Size!!" << std::endl;
		std::cout << "\t Not all objects included in MET calculation!" << std::endl;
		return;
        }
   }

}

void 
PhysicsObjectBuilder::CombinePhysicsObjects(TClonesArray* objects1, TClonesArray* objects2, 
	TClonesArray* combined)
{
   int iOut = 0;
   int outSize = combined->GetSize();

   int nObjects1s = objects1->GetEntries();
   for(int iObjects1=0; iObjects1 < nObjects1s; iObjects1++)
   {
	TParticle* tmpParticle = (TParticle*) (objects1->At(iObjects1));
	new(combined->operator[](iOut)) TParticle(*tmpParticle);
	iOut++;
	if(iOut>=outSize)
        {
		std::cout << "Error: N Physics Objects > combined objects TClonesArray Size!!" << std::endl;
		std::cout << "\t Not all objects included in MET calculation!" << std::endl;
		return;
        }
   }

   int nObjects2s = objects2->GetEntries();
   for(int iObjects2=0; iObjects2 < nObjects2s; iObjects2++)
   {
	TParticle* tmpParticle = (TParticle*) (objects2->At(iObjects2));
	new(combined->operator[](iOut)) TParticle(*tmpParticle);
	iOut++;
	if(iOut>=outSize)
        {
		std::cout << "Error: N Physics Objects > combined objects TClonesArray Size!!" << std::endl;
		std::cout << "\t Not all objects included in MET calculation!" << std::endl;
		return;
        }
   }
}


void 
PhysicsObjectBuilder::GetNConstituents(const fastjet::PseudoJet& jet, int& N,
	int& nCharged, int& nHadron) //const
{
	    std::vector<fastjet::PseudoJet> jetSubParticles = jet.constituents();
	    std::vector<fastjet::PseudoJet>::const_iterator jetSubPart;
	    for (jetSubPart = jetSubParticles.begin(); 
		jetSubPart != jetSubParticles.end(); jetSubPart++)
	    {
	      if(jetSubPart->has_constituents())
	      {
		int nConstit = jetSubPart->constituents().size();
		if(nConstit == 0)
	      		std::cout << "Warning: (Sub-)Jet with 0 Constituents!!" << std::endl;
		if(nConstit == 1)
		{
		      N++;
		      int pdg = abs(jetSubPart->user_index());
		      if (pdg != 11 && pdg != 13 && pdg != 22)
			nHadron++;
		      if (GetChargeFromPDG(pdg) != 0)
			nCharged++;
		}
		else
		{
			int subN=0;
			int subNCharged=0;
			int subNHadron=0;
			GetNConstituents(*jetSubPart, subN, subNCharged, subNHadron);
			N += subN;
			nCharged += subNCharged;
			nHadron += subNHadron;
		}
	      }
	      else
	      {
	      		//std::cout << "Info: Subjet with no constituents" << std::endl;
		      N++;
		      int pdg = abs(jetSubPart->user_index());
		      if (pdg != 11 && pdg != 13 && pdg != 22)
			nHadron++;
		      if (GetChargeFromPDG(pdg) != 0)
			nCharged++;
	      }
	    }

}

int 
PhysicsObjectBuilder::GetChargeFromPDG(const int pdg) const
{
  TParticlePDG* pdgPart = databasePDG->GetParticle(pdg);
  if(pdgPart == NULL)
  {
    std::cout << "Warning: PDGID "
       <<pdg<<" not found in TDatabasePDG," <<
       " assuming uncharged for Iso and jetID"<< std::endl;
    return 0;
  }
  return pdgPart->Charge();
}

} //namespace SPODS
