// SPODS--Simple Physics Object based Detector Simulation
// Copyright (C) 2011, 2012, 2013 Justin Hugon

#include <TSystem.h>
#include <TROOT.h>
#include <TClonesArray.h>
#include <TParticle.h>
#include <TFile.h>
#include <TTree.h>

#include <iostream>
#include <sstream>
#include <string>
#include <vector>

#include "HepMCImporter.h"
#include "PhysicsObjectBuilder.h"
#include "PhysicsObjectFunctions.h"
#include "POUtilities.h"
#include "YamlReader.h"
#include "YamlConfig.h"

int main(int argc, char *argv[])
{
  std::cout << "Starting SPODS" << std::endl;
  if (argc != 4) 
  {
	std::cout << "SPODS requires 3 arguments: " << std::endl
	<< "SPODS <config.yaml> <inFileName.hepmc> <outfilename.root>" << std::endl;
	return 1;
  }
  std::string configFileName = argv[1];
  std::string inFileName = argv[2];
  std::string outFileName = argv[3];

  std::cout << "Running Config:      " << configFileName << std::endl;
  std::cout << "Running Input File:  " << inFileName << std::endl;
  std::cout << "Running Output File: " << outFileName << std::endl;

  SPODS::YamlReader sillyReader(configFileName); //Hack b/c linking isn't working
  SPODS::YamlConfig * config = new SPODS::YamlConfig(configFileName);
  config->Print();
  //stringstream configStream;
  //config->Print(configStream);
  //TString configString(configStream.str().c_str());

  TFile* f = new TFile(outFileName.c_str(),"RECREATE");
  TTree * tree = new TTree("tree","tree");

  TClonesArray * allParticles = new TClonesArray("TParticle",5000);
  TClonesArray * particles = new TClonesArray("TParticle",config->maxGenParticles);
  TClonesArray * electrons = new TClonesArray("TParticle",config->maxElectrons);
  TClonesArray * muons = new TClonesArray("TParticle",config->maxMuons);
  TClonesArray * photons = new TClonesArray("TParticle",config->maxPhotons);
  TClonesArray * jets = new TClonesArray("TParticle",config->maxJets);
  TClonesArray * combinedObjects = new TClonesArray("TParticle",5000);
  double met=0.0;
  double metPhi=0.0;
  double sumPt=0.0;
  double trueMet=0.0;
  double trueMetPhi=0.0;
  double trueSumPt=0.0;
  double weight=0.0;
  int nMPI=0.0;
  tree->Branch("genParticles",particles);
  tree->Branch("electrons",electrons);
  tree->Branch("muons",muons);
  tree->Branch("photons",photons);
  tree->Branch("jets",jets);
  tree->Branch("met",&met,"met/D");
  tree->Branch("metPhi",&metPhi,"metPhi/D");
  tree->Branch("sumPt",&sumPt,"sumPt/D");
  tree->Branch("trueMet",&trueMet,"trueMet/D");
  tree->Branch("trueMetPhi",&trueMetPhi,"trueMetPhi/D");
  tree->Branch("trueSumPt",&trueSumPt,"trueSumPt/D");
  tree->Branch("weight",&weight,"weight/D");
  tree->Branch("nMPI",&nMPI,"nMPI/I");

  SPODS::HepMCImporter myImporter(inFileName);
  SPODS::PhysicsObjectBuilder myBuilder(config);
  int iEvents = 0;
  bool moreEvents;
  for(unsigned iEvent = 0; iEvent < config->maxEvents; iEvent++)
  {
	myImporter.GenerateEvent();
	moreEvents = myImporter.Done();
    if (!moreEvents)
        break;
	if (iEvent % 1000 == 0)
		std::cout << "Event: " << iEvent << std::endl;

    nMPI = myImporter.GetNMPI();
    weight = myImporter.GetWeight();

	int nParts = myImporter.ImportParticles(particles);
	int nAllParts = myImporter.ImportParticles(allParticles,"All");

	myBuilder.LoadGenParticles(particles);
	myBuilder.MakeElectrons(electrons);
	myBuilder.MakeMuons(muons);
	myBuilder.MakePhotons(photons);
	myBuilder.MakeJets(jets);

	myBuilder.MakeIso(muons,config->muonIsoType,config->muonIsoConeSize,
		config->muonIsoMinPt, "muon");
	myBuilder.MakeIso(electrons,config->electronIsoType,config->electronIsoConeSize,
		config->electronIsoMinPt, "electron");
	myBuilder.MakeIso(photons,config->photonIsoType,config->photonIsoConeSize,
		config->photonIsoMinPt, "photon");

   	//myBuilder.CombinePhysicsObjects(electrons, muons, photons, jets, combinedObjects);
   	myBuilder.CombinePhysicsObjects(muons, jets, combinedObjects);
	myBuilder.MakeMet(combinedObjects,met,metPhi,sumPt,trueMet, trueMetPhi, trueSumPt);

	myBuilder.BTagJets(jets,allParticles);

	removeLowPt(electrons, config->ptElectronMin);
	removeLowPt(muons, config->ptMuonMin);
	removeLowPt(photons, config->ptPhotonMin);
	removeLowPt(jets, config->ptJetMin);

	tree->Fill();
  }

  f->cd();
  tree->Write();

  delete allParticles;
  delete particles;
  delete electrons;
  delete muons;
  delete photons;
  delete jets;
  delete combinedObjects;
  delete tree;
  delete f;

  delete config;

  std::cout << "SPODS Done" << std::endl;
  return 0;
}
