
// SPODS--Simple Physics Object based Detector Simulation
// Copyright (C) 2011, 2012, 2013 Justin Hugon

#include <YamlConfig.h>
#include <YamlReader.h>
#include <stdlib.h>
#include <iostream>

namespace SPODS
{
YamlConfig::YamlConfig(std::string configFileName)
{
	YamlReader yReader(configFileName);

	maxEvents = atoi(yReader.GetValue("maxEvents").c_str());
	maxGenParticles = atoi(yReader.GetValue("maxGenParticles").c_str());
	maxElectrons = atoi(yReader.GetValue("maxElectrons").c_str());
	maxMuons = atoi(yReader.GetValue("maxMuons").c_str());
	maxPhotons = atoi(yReader.GetValue("maxPhotons").c_str());
	maxJets = atoi(yReader.GetValue("maxJets").c_str());

	etaElectronMax = atof(yReader.GetValue("etaElectronMax").c_str());
	etaMuonMax = atof(yReader.GetValue("etaMuonMax").c_str());
	etaPhotonMax = atof(yReader.GetValue("etaPhotonMax").c_str());
	etaJetMax = atof(yReader.GetValue("etaJetMax").c_str());
	etaMetMax = atof(yReader.GetValue("etaMetMax").c_str());

	ptElectronMin = atof(yReader.GetValue("ptElectronMin").c_str());
	ptMuonMin = atof(yReader.GetValue("ptMuonMin").c_str());
	ptPhotonMin = atof(yReader.GetValue("ptPhotonMin").c_str());
	ptJetMin = atof(yReader.GetValue("ptJetMin").c_str());

	jetAlg = yReader.GetValue("jetAlg");
	jetAlgR = atof(yReader.GetValue("jetAlgR").c_str());
	jetVetoNPartMin = atoi(yReader.GetValue("jetVetoNPartMin").c_str());
	jetVetoNChargMin = atoi(yReader.GetValue("jetVetoNChargMin").c_str());
	jetVetoNHadMin = atoi(yReader.GetValue("jetVetoNHadMin").c_str());

	muonIsoType = yReader.GetValue("muonIsoType");
	muonIsoConeSize = atof(yReader.GetValue("muonIsoConeSize").c_str());
	muonIsoMinPt = atof(yReader.GetValue("muonIsoMinPt").c_str());
	electronIsoType = yReader.GetValue("electronIsoType");
	electronIsoConeSize = atof(yReader.GetValue("electronIsoConeSize").c_str());
	electronIsoMinPt = atof(yReader.GetValue("electronIsoMinPt").c_str());
	photonIsoType = yReader.GetValue("photonIsoType");
	photonIsoConeSize = atof(yReader.GetValue("photonIsoConeSize").c_str());
	photonIsoMinPt = atof(yReader.GetValue("photonIsoMinPt").c_str());
	if(electronIsoType != "trk" && electronIsoType != "calo" &&
			electronIsoType != "combined")
	{
		std::cout << "Error: YamlConfig: electronIsoType is '"<<electronIsoType<<
				"' must be trk, calo, or combined\n";
		throw;
	}
	if(muonIsoType != "trk" && muonIsoType != "calo" &&
			muonIsoType != "combined")
	{
		std::cout << "Error: YamlConfig: muonIsoType is '"<<muonIsoType<<
				"' must be trk, calo, or combined\n";
		throw;
	}
	if(photonIsoType != "trk" && photonIsoType != "calo" &&
			photonIsoType != "combined")
	{
		std::cout << "Error: YamlConfig: photonIsoType is '"<<photonIsoType<<
				"' must be trk, calo, or combined\n";
		throw;
	}

	bTagEff = atof(yReader.GetValue("bTagEff").c_str());
	bTagMistagRate = atof(yReader.GetValue("bTagMistagRate").c_str());

	muonResEtaRanges = yReader.GetVectorOfPairs("muonResEtaRanges");
	muonResConstTerm = yReader.GetVectorOfFloats("muonResConstTerm");
	muonResSqrtTerm = yReader.GetVectorOfFloats("muonResSqrtTerm");
	muonResHighPtCut = yReader.GetVectorOfFloats("muonResHighPtCut");
	muonResHighPtConstTerm = yReader.GetVectorOfFloats("muonResHighPtConstTerm");
	muonResHighPtSqrtTerm = yReader.GetVectorOfFloats("muonResHighPtSqrtTerm");

	photonResEtaRanges = yReader.GetVectorOfPairs("photonResEtaRanges");
	photonResConstTerm = yReader.GetVectorOfFloats("photonResConstTerm");
	photonResOneOverETerm = yReader.GetVectorOfFloats("photonResOneOverETerm");
	photonResOneOverSqrtETerm = yReader.GetVectorOfFloats("photonResOneOverSqrtETerm");

	electronResPtCut = atof(yReader.GetValue("electronResPtCut").c_str());
	electronResConstantTerm = atof(yReader.GetValue("electronResConstantTerm").c_str());

	jetResPtCut = yReader.GetVectorOfFloats("jetResPtCut");
	jetResEtaRanges = yReader.GetVectorOfPairs("jetResEtaRanges");
	jetResConstTerm = yReader.GetVectorOfFloats("jetResConstTerm");
	jetResOneOverETerm = yReader.GetVectorOfFloats("jetResOneOverETerm");
	jetResOneOverSqrtETerm = yReader.GetVectorOfFloats("jetResOneOverSqrtETerm");

}


void 
YamlConfig::PrintVector(const std::vector<float> vec) const
{
  std::cout << "[";
  std::vector<float>::const_iterator iter = vec.begin();
  for(;iter != vec.end();iter++)
	std::cout << *iter << ", ";

  std::cout << "]\n";
}

void 
YamlConfig::PrintVectorOfPairs(const std::vector<std::pair<float,float> > vec) const
{
  std::cout << "[";
  std::vector< std::pair<float,float> >::const_iterator iter = vec.begin();
  for(;iter != vec.end();iter++)
  {
	float first = iter->first;
	float second = iter->second;
	std::cout << "("<<first << ","<<second << "),";
  }

  std::cout << "]\n";
}

void
YamlConfig::Print(std::ostream& ost) const
{
  ost << "\n###########################\n" << std::endl;
  ost << "SPODS Config:\n" << std::endl;

  ost << "maxEvents: " <<maxEvents<<  std::endl;

  ost << std::endl;

  ost << "maxGenParticles: " <<maxGenParticles<<  std::endl;
  ost << "maxElectrons: " <<maxElectrons<<  std::endl;
  ost << "maxMuons: " <<maxMuons<<  std::endl;
  ost << "maxPhotons: " <<maxPhotons<<  std::endl;
  ost << "maxJets: " <<maxJets<<  std::endl;

  ost << std::endl;

  ost << "etaElectronMax: " <<etaElectronMax<<  std::endl;
  ost << "etaMuonMax: " <<etaMuonMax<<  std::endl;
  ost << "etaPhotonMax: " <<etaPhotonMax<<  std::endl;
  ost << "etaJetMax: " <<etaJetMax<<  std::endl;
  ost << "etaMetMax: " <<etaMetMax<<  std::endl;

  ost << std::endl;

  ost << "ptElectronMin: " <<ptElectronMin<<  std::endl;
  ost << "ptMuonMin: " <<ptMuonMin<<  std::endl;
  ost << "ptPhotonMin: " <<ptPhotonMin<<  std::endl;
  ost << "ptJetMin: " <<ptJetMin<<  std::endl;

  ost << std::endl;

  ost << "jetAlg: " <<jetAlg<<  std::endl;
  ost << "jetAlgR: " <<jetAlgR<<  std::endl;
  ost << "jetVetoNPartMin: " <<jetVetoNPartMin<<  std::endl;
  ost << "jetVetoNChargMin: " <<jetVetoNChargMin<<  std::endl;
  ost << "jetVetoNHadMin: " <<jetVetoNHadMin<<  std::endl;

  ost << std::endl;

  ost << "muonIsoType: " <<muonIsoType<<  std::endl;
  ost << "muonIsoConeSize: " <<muonIsoConeSize<<  std::endl;
  ost << "muonIsoMinPt: " <<muonIsoMinPt<<  std::endl;
  ost << "electronIsoType: " <<electronIsoType<<  std::endl;
  ost << "electronIsoConeSize: " <<electronIsoConeSize<<  std::endl;
  ost << "electronIsoMinPt: " <<electronIsoMinPt<<  std::endl;
  ost << "photonIsoType: " <<photonIsoType<<  std::endl;
  ost << "photonIsoConeSize: " <<photonIsoConeSize<<  std::endl;
  ost << "photonIsoMinPt: " <<photonIsoMinPt<<  std::endl;

  ost << std::endl;

  ost << "bTagEff: " <<bTagEff<<  std::endl;
  ost << "bTagMistagRate: " <<bTagMistagRate<<  std::endl;

  ost << "\n###########################" << std::endl;
  ost << "SPODS Resolution Config\n" << std::endl;

  ost << "muonResEtaRanges: ";
  PrintVectorOfPairs(muonResEtaRanges);
  ost << "muonResConstTerm: ";
  PrintVector(muonResConstTerm);
  ost << "muonResSqrtTerm: ";
  PrintVector(muonResSqrtTerm);
  ost << "muonResHighPtCut: ";
  PrintVector(muonResHighPtCut);
  ost << "muonResHighPtConstTerm: ";
  PrintVector(muonResHighPtConstTerm);
  ost << "muonResHighPtSqrtTerm: ";
  PrintVector(muonResHighPtSqrtTerm);

  ost << std::endl;
  
  ost << "electronResPtCut: " << electronResPtCut << std::endl;
  ost << "electronResConstantTerm: " << electronResConstantTerm << std::endl;

  ost << std::endl;
  
  ost << "photonResEtaRanges: ";
  PrintVectorOfPairs(photonResEtaRanges);
  ost << "photonResConstTerm: ";
  PrintVector(photonResConstTerm);
  ost << "photonResOneOverETerm: ";
  PrintVector(photonResOneOverETerm);
  ost << "photonResOneOverSqrtETerm: ";
  PrintVector(photonResOneOverSqrtETerm);

  ost << std::endl;
  
  ost << "jetResEtaRanges: ";
  PrintVectorOfPairs(jetResEtaRanges);
  ost << "jetResConstTerm: ";
  PrintVector(jetResConstTerm);
  ost << "jetResOneOverETerm: ";
  PrintVector(jetResOneOverETerm);
  ost << "jetResOneOverSqrtETerm: ";
  PrintVector(jetResOneOverSqrtETerm);
  
  ost << "\n###########################\n" << std::endl;
}

}

