// SPODS--Simple Physics Object based Detector Simulation
// Copyright (C) 2011, 2012, 2013 Justin Hugon

// Modified from PYTHIA 8 Example:

////// main21.cc is a part of the PYTHIA event generator.
////// Copyright (C) 2011 Torbjorn Sjostrand.
////// PYTHIA is licenced under the GNU GPL version 2, see COPYING for details.
////// Please respect the MCnet Guidelines, see GUIDELINES for details.
////
////// This is a simple test program. 
////// It illustrates how to feed in a single particle 
////// or a toy parton-level configurations.

#include "Pythia.h"
#include "HepMCInterface.h"

#include "HepMC/GenEvent.h"
#include "HepMC/IO_GenEvent.h"
#include "TRandom3.h"
#include "TLorentzVector.h"
#include "TMath.h"
#include <iostream>

using namespace Pythia8; 

  double ptMin = 100.0;
  double ptMax = 100.1;
  double etaMin = 0.0;
  double etaMax = 5.0;
  // Set number of events to generate and to list.
  int nEvent = 100000;

//==========================================================================

// Single-particle gun. The particle must be a colour singlet.
// Input: flavour, energy, direction (theta, phi).
// If theta < 0 then random choice over solid angle. 

void fillParticle(int id, double ptMin, double ptMax, double etaMin, double etaMax,
		  Event& event, ParticleData& pdt, TRandom& random) {

  // Reset event record to allow for new event.
  event.reset();

  double pt = ptMin + (ptMax-ptMin)*random.Rndm();
  double phi = 2*TMath::Pi()*random.Rndm();
  double eta = etaMin + (etaMax-etaMin)*random.Rndm();

  // Select particle mass; where relevant according to Breit-Wigner. 
  double mm = pdt.mass(id);
  TLorentzVector vec;
  vec.SetPtEtaPhiM(pt,eta,phi, mm);

  // Store the particle in the event record.
  event.append( id, 1, 0, 0, vec.Px(), vec.Py(), 
    vec.Pz(), vec.E(), mm); 
}

//==========================================================================

// Simple method to do the filling of partons into the event record.

void fillPartons(int id, double ptMin, double ptMax, double etaMin, double etaMax,
		  Event& event, ParticleData& pdt, TRandom& random) {

  // Reset event record to allow for new event.
  event.reset();

  double pt = ptMin + (ptMax-ptMin)*random.Rndm();
  double phi = 2*TMath::Pi()*random.Rndm();
  double eta = etaMin + (etaMax-etaMin)*random.Rndm();

  // Select particle mass; where relevant according to Breit-Wigner. 
  double mm = pdt.mass(id);
  TLorentzVector vec;
  vec.SetPtEtaPhiM(pt,eta,phi, mm);

  // Store the particle in the event record.
  event.append( id, 23, 101, 0, vec.Px(), vec.Py(), 
    vec.Pz(), vec.E(), mm); 
  event.append( -id, 23, 0, 101, -vec.Px(), -vec.Py(), 
    -vec.Pz(), vec.E(), mm); 

}

void fillPartonsLeptons(int id, double ptMin, double ptMax, double etaMin, double etaMax,
		  Event& event, ParticleData& pdt, TRandom& random) {

  // Reset event record to allow for new event.
  event.reset();

  double pt = ptMin + (ptMax-ptMin)*random.Rndm();
  double phi = 2*TMath::Pi()*random.Rndm();
  double eta = etaMin + (etaMax-etaMin)*random.Rndm();

  // Select particle mass; where relevant according to Breit-Wigner. 
  double mm = pdt.mass(id);
  TLorentzVector vec;
  vec.SetPtEtaPhiM(pt,eta,phi, mm);
  TLorentzVector leptonVec = vec;
  leptonVec *= 1.0+random.Gaus(0.0,0.1);

  // Store the particle in the event record.
  event.append( id, 23, 101, 0, vec.Px(), vec.Py(), 
    vec.Pz(), vec.E(), mm); 
  event.append( -id, 23, 0, 101, -vec.Px(), -vec.Py(), 
    -vec.Pz(), vec.E(), mm); 
  event.append( 13, 1, 0, 0, leptonVec.Px(), leptonVec.Py(), 
    leptonVec.Pz(), leptonVec.E(), pdt.mass(13)); 
  event.append( -11, 1, 0, 0, -leptonVec.Px(), -leptonVec.Py(), 
    -leptonVec.Pz(), leptonVec.E(), pdt.mass(11)); 

}

//==========================================================================

int main(int argc, char *argv[])
{
  // Check that correct number of command-line arguments
  if (argc != 3) {
    cerr << " Unexpected number of command-line arguments. \n You are"
         << " expected to provide one pdg id (or 0 for all types, 999 for leptons overlapped with jets) and one output file name. \n"
         << " Program stopped! " << endl;
    return 1;
  }

  TRandom3 random(1234);
  // Confirm that external files will be used for input and output.
  cout << " Particle Generated " << argv[1] << endl;
  cout << " HepMC events will be written to file " << argv[2] << endl;

  // Interface for conversion from Pythia8::Event to HepMC one. 
  HepMC::I_Pythia8 ToHepMC;
  //  ToHepMC.set_crash_on_problem();

  // Specify file where HepMC events will be stored.
  HepMC::IO_GenEvent ascii_io(argv[2], std::ios::out);

  // Set particle species and energy for single-particle gun.
  int    idGun = 0;
  bool doAll=false;
  bool nonIsoTest=false;
  if(atoi(argv[1]) == 0)
	doAll = true;
  else if(atoi(argv[1]) == 999)
	nonIsoTest = true;
  else
    idGun = atoi(argv[1]);

  int type =0;
  if((idGun <10 && idGun >-10) || idGun==21)
	type=1;


  // Generator; shorthand for event and particleData.                           
  Pythia pythia;  
  Event& event      = pythia.event;
  ParticleData& pdt = pythia.particleData;

  // Key requirement: switch off ProcessLevel, and thereby also PartonLevel.
  pythia.readString("ProcessLevel:all = off");
  pythia.readString("Main:timesAllowErrors = 100000");

  // Optionally switch off decays.
  //pythia.readString("HadronLevel:Decay = off");

  // Provide printout of initial information.        
  pythia.settings.listChanged();
 
  // Initialize.
  pythia.init();
  
  int types[5] = {1,5,11,13,22};
  // Begin of event loop.
  for (int iEvent = 0; iEvent < nEvent; ++iEvent) {
    if (iEvent%(max(1,nEvent/20)) == 0) cout << " Now begin event " 
      << iEvent << endl;

    if (doAll)
    {
	unsigned id = types[iEvent % 5];
        if(id == 1)
		fillPartons( id, ptMin, ptMax, etaMin, etaMax, event, pdt, random);
        else if(id == 5)
		fillPartons( id, ptMin, ptMax, etaMin, etaMax, event, pdt, random);
	else
      		fillParticle( id, ptMin, ptMax, etaMin, etaMax, event, pdt, random);
    }
    else if (nonIsoTest)
    {
	fillPartonsLeptons(2,ptMin,ptMax,etaMin,etaMax,event,pdt,random);
    }
    else
    {
      // Set up single particle, with random direction in solid angle.
      if (type == 0) fillParticle( idGun, ptMin, ptMax, etaMin, etaMax, event, pdt, random);

      // Set up parton-level configuration.
      else fillPartons( idGun, ptMin, ptMax, etaMin, etaMax, event, pdt, random);
    }

    // Generate events. Quit if failure.
    if (!pythia.next()) {
      cout << " Event generation aborted prematurely, owing to error!\n"; 
      break;
    }

    // Construct new empty HepMC event. Form with arguments is only
    // meaningful for HepMC 2.04 onwards, and even then unnecessary  
    // if HepMC was built with GeV and mm as units from the onset. 
    HepMC::GenEvent* hepmcevt = new HepMC::GenEvent();
    //HepMC::GenEvent* hepmcevt = new HepMC::GenEvent(
    //  HepMC::Units::GEV, HepMC::Units::MM); 

    // Fill HepMC event, including PDF info.
    ToHepMC.fill_next_event( pythia, hepmcevt );
    // This alternative older method fills event, without PDF info.
    // ToHepMC.fill_next_event( pythia.event, hepmcevt );

    // Write the HepMC event to file. Done with it.
    ascii_io << hepmcevt;
    delete hepmcevt;
                       
  // End of event loop.
  }                                           

  // Print statistics, histograms and done.
  pythia.statistics();

  // Done.
  return 0;
}
