#!/usr/bin/python

from hepmcParticleGun import *
from subprocess import check_call, CalledProcessError
import os
import ROOT as root
import sys
import linecache
import re

## Runs a series of tests on SPODS
CONFIGFILE="noSmearingConfig.yaml"
#CONFIGFILE="cmsConfig.yaml"

root.gROOT.SetBatch(1)
root.gStyle.SetOptStat(0)

debug=False
if len(sys.argv)>1:
  if sys.argv[1] == "--debug" or sys.argv[1] == "-debug":
    debug=True

def divideYValByXVal(hist):
    nBinsX = hist.GetXaxis().GetNbins()
    for iBinX in range(1,nBinsX+1):
      binVal = hist.GetBinContent(iBinX)
      binErrVal = hist.GetBinError(iBinX)
      xVal = hist.GetXaxis().GetBinCenter(iBinX)
      hist.SetBinContent(iBinX,binVal/xVal)
      hist.SetBinError(iBinX,binErrVal/xVal)

##########################################################################

class TestParticle:
  def __init__(self,pdgid,nParticles,nEvents,debug):
    self.pdgid = pdgid
    self.debug = debug
    self.nEvents = nEvents 
    self.nParticles = nParticles
    self.efficiency = 0.0

  def run(self):
    infileName = "/tmp/spodsTest%i_%i.hepmc" % (self.pdgid,self.nParticles)
    outfileName = "/tmp/spodsTest%i_%i.root" % (self.pdgid,self.nParticles)
    logfileName = "/tmp/spodsTest%i_%i.log" % (self.pdgid,self.nParticles)
    logfile = open(logfileName,"w")

    hepmc = MakeHepMC()
    if self.pdgid == 13:
      hepmc.nMu = self.nParticles
      hepmc.nEl = 0
      hepmc.nPh = 0
      hepmc.nHad = 0
      hepmc.nJet = 0
    elif self.pdgid == 11:
      hepmc.nMu = 0
      hepmc.nEl = self.nParticles
      hepmc.nPh = 0
      hepmc.nHad = 0
      hepmc.nJet = 0
    elif self.pdgid == 22:
      hepmc.nMu = 0
      hepmc.nEl = 0
      hepmc.nPh = self.nParticles
      hepmc.nHad = 0
      hepmc.nJet = 0
    elif self.pdgid == 211:
      hepmc.nMu = 0
      hepmc.nEl = 0
      hepmc.nPh = 0
      hepmc.nHad = 2
      hepmc.nJet = self.nParticles
    hepmc.generate(self.nEvents)
    hepmc.writeFile(infileName)
    
    try:
      check_call(["../bin/SPODS",CONFIGFILE,infileName,outfileName],stdout=logfile,stderr=logfile)
    except CalledProcessError as e:
      print("Error: TestParticle pdgid %i: SPODS returns error code: %s" % (self.pdgid,e.returncode))
    logfile.close()

    self.passed = True
    tfile = root.TFile(outfileName)
    tree = tfile.Get("tree")
    nEntries = tree.GetEntries()
    nFound = 0.0
    if nEntries != self.nEvents:
        self.passed = False
        print "Failed Event Number Test: Input: %i Output: %i" %(self.nEvents, nEntries)
    else:
      for iEvt in range(nEntries):
        tree.GetEvent(iEvt)
        if abs(self.pdgid) == 13:
          nFound += tree.muons.GetEntries()
          if tree.muons.GetEntries() != self.nParticles:
            self.passed = False
            print "Failed Muon Number Test: nFound/nCreated = %i/%i" % (tree.muons.GetEntries(),self.nParticles)
        elif abs(self.pdgid) == 11:
          nFound += tree.electrons.GetEntries()
          if tree.electrons.GetEntries() != self.nParticles:
            self.passed = False
            print "Failed Electron Number Test: nFound/nCreated = %i/%i" % (tree.electrons.GetEntries(),self.nParticles)
        elif abs(self.pdgid) == 22: 
          nFound += tree.photons.GetEntries()
          if tree.photons.GetEntries() != self.nParticles:
            self.passed = False
            print "Failed Photon Number Test: nFound/nCreated = %i/%i" % (tree.photons.GetEntries(),self.nParticles)
        elif ((abs(self.pdgid) < 6 and self.pdgid != 0) or self.pdgid==21 or abs(self.pdgid)==211): 
          nFound += tree.jets.GetEntries()
          if tree.jets.GetEntries() != self.nParticles:
            self.passed = False
            #print "Failed Jet Number Test: nFound/nCreated = %i/%i" % (tree.jets.GetEntries(),self.nParticles)

    self.efficiency = nFound/self.nEvents/self.nParticles
    
    if not self.debug:
      os.remove(infileName)
      os.remove(outfileName)
      os.remove(logfileName)

  def report(self):
    passString = ""
    if self.passed:
        passString = "Passed"
    else:
        passString = "Failed!!!*********"
    
    if ((abs(self.pdgid) < 6 and self.pdgid != 0) or self.pdgid==21 or abs(self.pdgid)==211):
      effTarget = 1.0
      if self.nParticles>1:
        effTarget = 0.85
      if self.efficiency >= effTarget:
        print("Test: Particle Gun Jet (Overlapping Hadrons) Passed")
        self.passed = True
      else:
        print("Test: Particle Gun Jet (Overlapping Hadrons) Failed!!!******")
      print("  Efficiency: %.3f" % self.efficiency)
    else:
      print("Test: Particle Gun PDGID %i %s" % (self.pdgid,passString))
      print("  Efficiency: %.3f" % self.efficiency)
    return self.passed

##########################################################################

class ResolutionTests:

  def getConfigParam(self,configFileName,name,rangeParam=False,normalParam=False):
    result = []
    if rangeParam:
      iLine = 1
      while True:
        line = linecache.getline(configFileName,iLine)
        if line == "":
            break 
        if re.search(name,line):
          tmpILine = iLine + 1
          while True:
            tmpLine = linecache.getline(configFileName,tmpILine)
            match = re.match(r"^ - *\[ *([\d\.]+) *\, *([\d\.]+) *\].*",tmpLine)
            if not match:
                break
            result.append([float(match.group(1)),float(match.group(2))])
            tmpILine += 1
        iLine += 1
    elif normalParam:
      iLine = 1
      while True:
        line = linecache.getline(configFileName,iLine)
        if line == "":
            break
        match = re.search(name+" *: *([0-9\.]+)",line)
        if match:
            result.append(float(match.group(1)))
        iLine += 1
    else:
      iLine = 1
      while True:
        line = linecache.getline(configFileName,iLine)
        if line == "":
            break
        if re.search(name,line):
          tmpILine = iLine + 1
          while True:
            tmpLine = linecache.getline(configFileName,tmpILine)
            match = re.match(r"^ - *([0-9\.]*).*",tmpLine)
            if not match:
                break
            result.append(float(match.group(1)))
            tmpILine += 1
        iLine += 1
        
    #print name
    #print "result %s" % result
    return result

  def __init__(self,nParticles,nEvents,debug):
    configName = "cmsConfig.yaml"
    self.nParticles = nParticles
    self.nEvents = nEvents
    self.debug = debug
    self.ErrThreshold = 10.0

    self.photonResEtaRanges = self.getConfigParam(configName,"photonResEtaRanges",rangeParam=True)
    self.photonResConstTerm = self.getConfigParam(configName,"photonResConstTerm")
    self.photonResOneOverETerm = self.getConfigParam(configName,"photonResOneOverETerm")
    self.photonResOneOverSqrtETerm = self.getConfigParam(configName,"photonResOneOverSqrtETerm")

    self.jetResEtaRanges = self.getConfigParam(configName,"jetResEtaRanges",rangeParam=True)
    self.jetResConstTerm = self.getConfigParam(configName,"jetResConstTerm")
    self.jetResOneOverETerm = self.getConfigParam(configName,"jetResOneOverETerm")
    self.jetResOneOverSqrtETerm = self.getConfigParam(configName,"jetResOneOverSqrtETerm")

    self.electronResPtCut = self.getConfigParam(configName,"electronResPtCut",normalParam=True)
    self.electronResConstantTerm = self.getConfigParam(configName,"electronResConstantTerm",normalParam=True)

    self.muonResEtaRanges = self.getConfigParam(configName,"muonResEtaRanges",rangeParam=True)

    self.muonResConstTerm = self.getConfigParam(configName,"muonResConstTerm")
    self.muonResSqrtTerm = self.getConfigParam(configName,"muonResSqrtTerm")
    self.muonResHighPtConstTerm = self.getConfigParam(configName,"muonResHighPtConstTerm")
    self.muonResHighPtSqrtTerm = self.getConfigParam(configName,"muonResHighPtSqrtTerm")
    self.muonResHighPtCut = self.getConfigParam(configName,"muonResHighPtCut")

    if(len(self.photonResEtaRanges) != len(self.photonResConstTerm) or len(self.photonResEtaRanges) != len(self.photonResOneOverSqrtETerm) or len(self.photonResEtaRanges) != len(self.photonResOneOverETerm)):
        print("Error: photon config info doesn't have same number of Eta Ranges")
        sys.exit()
    if(len(self.jetResEtaRanges) != len(self.jetResConstTerm) or len(self.jetResEtaRanges) != len(self.jetResOneOverSqrtETerm) or len(self.jetResEtaRanges) != len(self.jetResOneOverETerm)):
        print("Error: jet config info doesn't have same number of Eta Ranges")
        sys.exit()
    if(len(self.muonResEtaRanges) != len(self.muonResConstTerm) or len(self.muonResEtaRanges) != len(self.muonResSqrtTerm) or len(self.muonResEtaRanges) != len(self.muonResHighPtConstTerm) or len(self.muonResEtaRanges) != len(self.muonResHighPtSqrtTerm) or len(self.muonResEtaRanges) != len(self.muonResHighPtCut)):
        print("Error: Muon config info doesn't have same number of Eta Ranges")
        sys.exit()

    self.resultsEl = []
    self.resultsPh = []
    self.resultsMu = []
    self.resultsJet = []
    self.resultTitlesPh = ["Const Term","1/E Term","1/sqrt(E) Term"]
    self.resultTitlesEl = ["Const Term","1/E Term","1/sqrt(E) Term","Pt Cut","Const Term (below Pt Cut)"]
    self.resultTitlesMu = ["High Pt Cut","Const Term","sqrt(p) Term","High Pt Const Term","High Pt sqrt(p) Term"]
    self.resultTitlesJet = ["Const Term","1/E Term","1/sqrt(E) Term"]

  def run(self):
    infileName = "/tmp/spodsTestRes_%i.hepmc" % (self.nParticles)
    outfileName = "/tmp/spodsTestRes_%i.root" % (self.nParticles)
    logfileName = "/tmp/spodsTestRes_%i.log" % (self.nParticles)
    logfile = open(logfileName,"w")

    hepmc = MakeHepMC()
    hepmc.nMu = self.nParticles
    hepmc.nEl = self.nParticles
    hepmc.nPh = self.nParticles
    hepmc.nHad = 0
    hepmc.generate(self.nEvents)
    hepmc.writeFile(infileName)
    
    try:
      check_call(["../bin/SPODS",CONFIGFILE,infileName,outfileName],stdout=logfile,stderr=logfile)
    except CalledProcessError as e:
      print("Error: ResolutionParticle: SPODS returns error code: %s" % (e.returncode))
    logfile.close()

    self.passed = 1000000
    tfile = root.TFile(outfileName)
    tree = tfile.Get("tree")
    nEntries = tree.GetEntries()
    if nEntries != self.nEvents:
        self.passed = 0
        print "Failed Event Number Test: Input: %i Output: %i" %(self.nEvents, nEntries)
    else:
        self.getRes(tree,11)
        self.getRes(tree,13)
        self.getRes(tree,22)

    if not self.debug:
      os.remove(infileName)
      os.remove(outfileName)
      os.remove(logfileName)

  def report(self):
    print("\nAll Values below are:\n(Fit Parameter Value - Correct Parameter Value) / Fit Parameter Error:")

    failAll = False
    if self.passed == 0:
        failAll = True
    self.passed = 0
    nTried = 0
    print("\nElectron Resolution Parameters:")
    for iEta in range(len(self.photonResEtaRanges)):
      etaRanges = self.photonResEtaRanges
      minEta = etaRanges[iEta][0]
      maxEta = etaRanges[iEta][1]
      if(minEta==0.0):
        etaCutDrawString = "\nEta Range: |eta|<%.1f" % (maxEta)
      else:
        etaCutDrawString = "\nEta Range: %.1f<|eta|<%.1f" % (minEta,maxEta)
      print(etaCutDrawString)
      for i in range(len(self.resultsEl[iEta])):
        if re.search(r"[Cc]ut",self.resultTitlesEl[i]):
            continue
        result = self.resultsEl[iEta][i]
        print("Parameter %i: %.2e,    %s" % (i,result,self.resultTitlesEl[i]))
        if abs(result) > self.ErrThreshold:
            print("  Error Larger than Fit Error!! Fail Test!!!")
        else:
            self.passed += 1
        nTried+=1
    print("\nPhoton Resolution Parameters:")
    for iEta in range(len(self.photonResEtaRanges)):
      etaRanges = self.photonResEtaRanges
      minEta = etaRanges[iEta][0]
      maxEta = etaRanges[iEta][1]
      if(minEta==0.0):
        etaCutDrawString = "\nEta Range: |eta|<%.1f" % (maxEta)
      else:
        etaCutDrawString = "\nEta Range: %.1f<|eta|<%.1f" % (minEta,maxEta)
      print(etaCutDrawString)
      for i in range(len(self.resultsPh[iEta])):
        if re.search(r"[Cc]ut",self.resultTitlesPh[i]):
            continue
        result = self.resultsPh[iEta][i]
        print("Parameter %i: %.2e,    %s" % (i,result,self.resultTitlesPh[i]))
        if abs(result) > self.ErrThreshold:
            print("  Error Larger than Fit Error!! Fail Test!!!")
        else:
            self.passed += 1
        nTried+=1
    print("\nMuon Resolution Parameters:")
    for iEta in range(len(self.muonResEtaRanges)):
      etaRanges = self.muonResEtaRanges
      minEta = etaRanges[iEta][0]
      maxEta = etaRanges[iEta][1]
      if(minEta==0.0):
        etaCutDrawString = "\nEta Range: |eta|<%.1f" % (maxEta)
      else:
        etaCutDrawString = "\nEta Range: %.1f<|eta|<%.1f" % (minEta,maxEta)
      print(etaCutDrawString)
      for i in range(len(self.resultsMu[iEta])):
        if re.search(r"[Cc]ut",self.resultTitlesMu[i]):
            continue
        result = self.resultsMu[iEta][i]
        print("Parameter %i: %.2e,    %s" % (i,result,self.resultTitlesMu[i]))
        if abs(result) > self.ErrThreshold:
            print("  Error Larger than Fit Error!! Fail Test!!!")
        else:
            self.passed += 1
        nTried+=1

    if failAll:
      self.passed = 0

    passString = ""
    if self.passed == nTried:
        passString = "Passed"
    else:
        passString = "Failed!!!*********"
    
    print("\nTest: Resolutions %s\n    Passed %i/%i" % (passString,self.passed,nTried))
    return self.passed, nTried

  def getRes(self,tree,pdgid):
    name = "jets"
    nameShort = "jet"
    nameLatex = "Jet"
    energyLabel = "E_{T}"
    etaRanges = []
    tlatex = root.TLatex()
    tlatex.SetNDC()
    c1 = root.TCanvas("c1")
    if pdgid == 11:
        name = "electrons"
        nameShort = "el"
        nameLatex = "Electron"
        energyLabel = "p_{T}"
        etaRanges = self.photonResEtaRanges
    elif pdgid == 13:
        name = "muons"
        nameShort = "mu"
        nameLatex = "#mu"
        energyLabel = "p_{T}"
        etaRanges = self.muonResEtaRanges
    elif pdgid == 22:
        name = "photons"
        nameShort = "ph"
        nameLatex = "#gamma"
        energyLabel = "E_{T}"
        etaRanges = self.photonResEtaRanges
    for iEtaRange in range(len(etaRanges)):
      minEta = etaRanges[iEtaRange][0]
      maxEta = etaRanges[iEtaRange][1]
      etaCutString = "%.1f<=%s.Eta() && %s.Eta()<=%.1f" % (minEta,name,name,maxEta)
      if(minEta==0.0):
        etaCutDrawString = "|#eta|<%.1f" % (maxEta)
      else:
        etaCutDrawString = "%.1f<|#eta|<%.1f" % (minEta,maxEta)
      if len(etaRanges)>1:
        etaCutShortString = "_%.1feta%.1f_" % (minEta,maxEta)
      else:
        etaCutShortString = ""
      tree.Draw("1.0/"+name+"->GetCalcMass():"+name+"->Pt()*"+name+"->GetCalcMass()",etaCutString,"colz");
      htemp = root.gPad.GetPrimitive("htemp");
      if htemp != None:
        htemp.SetTitle(nameLatex+" Resolution");
        htemp.GetXaxis().SetTitle(nameLatex+" "+energyLabel+"^{Gen}");
        if(pdgid != 13):
          htemp.GetYaxis().SetTitle(energyLabel+"^{Smeared}/"+energyLabel+"^{Gen}");
          htemp.Rebin2D(2,1)
        htemp.FitSlicesY()
        htemp_2 = root.gDirectory.Get("htemp_2");
        c1.Update();
        c1.SaveAs(nameShort+etaCutShortString+"Res2D.png");
        c1.Clear();
        htemp_2.SetTitle(nameLatex+" Resolution #sigma v. "+energyLabel);
        htemp_2.GetYaxis().SetTitle("#sigma(smeared/gen)");
        htemp_2.GetXaxis().SetRange(1,htemp_2.GetNbinsX()-3);
        htemp_2.Draw();
        correctFn = ""
        xmin = htemp_2.GetXaxis().GetXmin()
        xmax = htemp_2.GetXaxis().GetXmax()
        if pdgid == 22:
          correctFn = root.TF1("correctFn","sqrt(pow([0],2)+pow([1]/x,2)+pow([2],2)/x)",xmin,xmax)
          correctFn.SetParameter(0,self.photonResConstTerm[iEtaRange])
          correctFn.SetParameter(1,self.photonResOneOverETerm[iEtaRange])
          correctFn.SetParameter(2,self.photonResOneOverSqrtETerm[iEtaRange])
        elif pdgid == 11:
          correctFn = root.TF1("correctFn","sqrt(pow([0],2)+pow([1]/x,2)+pow([2],2)/x)*(x>[3]) + (x<=[3])*[4]",xmin,xmax)
          correctFn.SetParameter(0,self.photonResConstTerm[iEtaRange])
          correctFn.SetParameter(1,self.photonResOneOverETerm[iEtaRange])
          correctFn.SetParameter(2,self.photonResOneOverSqrtETerm[iEtaRange])
          correctFn.FixParameter(3,self.electronResPtCut[iEtaRange])
          correctFn.SetParameter(4,self.electronResConstantTerm[iEtaRange])
        elif pdgid == 13:
          correctFn = root.TF1("correctFn","(x<=[0])*([1]+[2]*sqrt(x/1000.0)) + (x>[0])*([3]+[4]*sqrt(x/1000.0))",xmin,xmax)
          correctFn.FixParameter(0,self.muonResHighPtCut[iEtaRange])
          correctFn.SetParameter(1,self.muonResConstTerm[iEtaRange])
          correctFn.SetParameter(2,self.muonResSqrtTerm[iEtaRange])
          correctFn.SetParameter(3,self.muonResHighPtConstTerm[iEtaRange])
          correctFn.SetParameter(4,self.muonResHighPtSqrtTerm[iEtaRange])
        else:
          print("Error: ResTest.getRes() used on a not implemented pdgid: %i" % pdgid)
          sys.exit()
        #correctFn.SetLineStyle(2)
        fitFn = correctFn.Clone("correctFn")
        fitFn.SetLineColor(root.kBlue)
        htemp_2.Fit(fitFn,"Q")
        #htemp_2.Fit(fitFn,"QLME")
        chi2 = fitFn.GetChisquare();
        ndf = fitFn.GetNDF();
        tlatex.DrawLatex(0.6,0.85,"#chi^{2}/NDF %.2f/%.2f" % (chi2,ndf))
        tlatex.DrawLatex(0.3,0.85,etaCutDrawString)
        correctFn.Draw("same")
        c1.SaveAs(nameShort+etaCutShortString+"Res.png");

        ## Now get out fit parameters and compare to correct
        nPar = fitFn.GetNpar()
        parOverErrList = [9999.0]*nPar
        for iPar in range(nPar):
            if fitFn.GetParError(iPar) == 0.0:
                continue
            if chi2/ndf > 10:
                paramOverErr = -200000
            else:
              paramOverErr = (fitFn.GetParameter(iPar)-correctFn.GetParameter(iPar))/fitFn.GetParError(iPar)
            parOverErrList[iPar] = paramOverErr
        if pdgid == 22:
            self.resultsPh.append(parOverErrList)
        elif pdgid == 11:
            self.resultsEl.append(parOverErrList)
        elif pdgid == 13:
            self.resultsMu.append(parOverErrList)
            
      else:
        print "Error: can't get resolution histogram 'htemp'"

##########################################################################

class IsoAndBTagTests:
  def __init__(self,pdgid,nParticles,nEvents,isolatedOrB,debug):
    self.pdgid = pdgid
    self.debug = debug
    self.nEvents = nEvents 
    self.nParticles = nParticles
    self.efficiency = 0.0
    self.isolatedOrB = isolatedOrB
    if pdgid ==5:
      if isolatedOrB:
        self.testname = "b-Efficiency Test"
      else:
        self.testname = "b-Mistag Test"
    else:
      if isolatedOrB:
        self.testname = "Isolated Test"
      else:
        self.testname = "Anti-Isolated Test"

  def getConfigParam(self,configFileName,name):
      result = -99999999999999.9
      iLine = 1
      while True:
        line = linecache.getline(configFileName,iLine)
        if line == "":
            break
        match = re.search(name+" *: *([0-9\.]+)",line)
        if match:
            result = float(match.group(1))
        iLine += 1
      return result

  def run(self):
    isIsoStr = ""
    if pdgid ==5:
      isIsoStr = "bFake"
      if self.isolatedOrB:
        isIsoStr = "bEff"
    else:
      isIsoStr = "AntiIso"
      if self.isolatedOrB:
        isIsoStr = "Iso"
    infileName = "/tmp/spodsExtraTest%i%s_%i.hepmc" % (self.pdgid,isIsoStr,self.nParticles)
    outfileName = "/tmp/spodsExtraTest%i%s_%i.root" % (self.pdgid,isIsoStr,self.nParticles)
    logfileName = "/tmp/spodsExtraTest%i%s_%i.log" % (self.pdgid,isIsoStr,self.nParticles)
    logfile = open(logfileName,"w")

    hepmc = MakeHepMC()

    nHad = 1
    if self.isolatedOrB:
        nHad = 0

    if self.pdgid == 13:
      hepmc.nMu = self.nParticles
      hepmc.nEl = 0
      hepmc.nPh = 0
      hepmc.nHad = nHad
      hepmc.nJet = 0
    elif self.pdgid == 11:
      hepmc.nMu = 0
      hepmc.nEl = self.nParticles
      hepmc.nPh = 0
      hepmc.nHad = nHad
      hepmc.nJet = 0
    elif self.pdgid == 22:
      hepmc.nMu = 0
      hepmc.nEl = 0
      hepmc.nPh = self.nParticles
      hepmc.nHad = nHad
      hepmc.nJet = 0
    elif self.pdgid == 5:
      hepmc.nMu = 0
      hepmc.nEl = 0
      hepmc.nPh = 0
      hepmc.nHad = 2
      hepmc.nJet = self.nParticles
      if self.isolatedOrB:
        hepmc.bJets = True
    else:
      print("Error: ExtraTests can only test mu,e,photon, and b")
      sys.exit()

    if self.pdgid ==5:
       hepmc.generate(self.nEvents)
    else:
      if self.isolatedOrB:
        hepmc.generate(self.nEvents)
      else:
        hepmc.generateAntiIsolated(self.nEvents)
    hepmc.writeFile(infileName)
    
    try:
      check_call(["../bin/SPODS",CONFIGFILE,infileName,outfileName],stdout=logfile,stderr=logfile)
    except CalledProcessError as e:
      print("Error: ExtraTest pdgid %i: SPODS returns error code: %s" % (self.pdgid,e.returncode))
    logfile.close()

    self.passed = True
    tfile = root.TFile(outfileName)
    tree = tfile.Get("tree")
    nEntries = tree.GetEntries()
    nFound = 0.0
    nBJets = 0
    nJets = 0
    if nEntries != self.nEvents:
        self.passed = False
        print "ExtraTest: Failed Event Number Test: Input: %i Output: %i" %(self.nEvents, nEntries)
    else:
      for iEvt in range(nEntries):
        tree.GetEvent(iEvt)
        if abs(self.pdgid) == 13:
          for mu in tree.muons:
            relIsoError = mu.GetWeight()/mu.Pt()-nHad
            if  abs(relIsoError) > 0.01:
              self.passed = False
              print "Failed %s: Off by: %.2f%%" % (self.testname,relIsoError*100.0)
        elif abs(self.pdgid) == 11:
          for el in tree.electrons:
            relIsoError = el.GetWeight()/el.Pt()-nHad
            if  abs(relIsoError) > 0.01:
              self.passed = False
              print "Failed %s: Off by: %.2f%%" % (self.testname,relIsoError*100.0)
        elif abs(self.pdgid) == 22: 
          for ph in tree.photons:
            relIsoError = ph.GetWeight()/ph.Pt()-nHad
            if  abs(relIsoError) > 0.01:
              self.passed = False
              print "Failed %s: Off by: %.2f%%" % (self.testname,relIsoError*100.0)

        elif self.pdgid == 5:
          for jet in tree.jets:
            iDaughter = jet.GetLastDaughter()
            bTag = False
            if iDaughter == 1:
                bTag = True
            elif iDaughter == 0:
                bTag = False
            else:
                print "Error: b-Tag not 0 or 1!!"
                self.passed = False
            if bTag:
                nBJets += 1
            nJets += 1

    if self.pdgid == 5:
        self.efficiency = float(nBJets)/nJets
        self.goal = 0.0
        if self.isolatedOrB:
          self.goal = self.getConfigParam(CONFIGFILE,"bTagEff")
        else:
          self.goal = self.getConfigParam(CONFIGFILE,"bTagMistagRate")
        if abs(self.efficiency-self.goal)>0.03:
            self.passed = False
        

    if not self.debug:
      os.remove(infileName)
      os.remove(outfileName)
      os.remove(logfileName)


  def report(self):
    passString = ""
    if self.passed:
        passString = "Passed"
    else:
        passString = "Failed!!!*********"
    
    print("Test: %s PDGID=%i %s" % (self.testname,self.pdgid,passString))
    if self.pdgid == 5:
        print("  Fraction of B-Jets: %.3f, goal: %.3f" % (self.efficiency,self.goal))
    return self.passed

##########################################################################

class TestMPIWeight:
  def __init__(self,mpi,weight,nEvents,debug):
    self.pdgid = 13
    self.debug = debug
    self.nEvents = nEvents 
    self.nParticles = 4
    self.efficiency = 0.0
    self.mpi = mpi
    self.weight = weight

  def run(self):
    infileName = "/tmp/spodsTestMPI%i_%i.hepmc" % (self.mpi,self.nParticles)
    outfileName = "/tmp/spodsTestMPI%i_%i.root" % (self.mpi,self.nParticles)
    logfileName = "/tmp/spodsTestMPI%i_%i.log" % (self.mpi,self.nParticles)
    logfile = open(logfileName,"w")

    hepmc = MakeHepMC()
    hepmc.mpi = self.mpi
    hepmc.weight = self.weight
    hepmc.generate(self.nEvents)
    hepmc.writeFile(infileName)
    
    try:
      check_call(["../bin/SPODS",CONFIGFILE,infileName,outfileName],stdout=logfile,stderr=logfile)
    except CalledProcessError as e:
      print("Error: TestMPIWeight MPI %i: SPODS returns error code: %s" % (self.mpi,e.returncode))
    logfile.close()

    self.passed = True
    tfile = root.TFile(outfileName)
    tree = tfile.Get("tree")
    nEntries = tree.GetEntries()
    nFound = 0.0
    if nEntries != self.nEvents:
        self.passed = False
        print "Failed Event Number Test: Input: %i Output: %i" %(self.nEvents, nEntries)
    else:
      for iEvt in range(nEntries):
        tree.GetEvent(iEvt)
        if tree.nMPI != self.mpi:
            self.passed = False
            print "Failed MPI Test: Found/Created = %i/%i" % (tree.nMPI,self.mpi)
        if self.weight == None:
          if tree.weight != 1.0:
            self.passed = False
            print "Failed Weight Test: Found/Created = %f/None" % (tree.weight)
        else:
          if tree.weight != self.weight:
            self.passed = False
            print "Failed Weight Test: Found/Created = %f/%f" % (tree.weight,self.weight)
        
    if not self.debug:
      os.remove(infileName)
      os.remove(outfileName)
      os.remove(logfileName)

  def report(self):
    passString = ""
    if self.passed:
        passString = "Passed"
    else:
        passString = "Failed!!!*********"
    
    print("Test: MPI %i Weight %s %s" % (self.mpi,str(self.weight),passString))
    return self.passed

##########################################################################
##########################################################################
##########################################################################

if __name__=="__main__":
  pdgids = [11,13,22,211]
  singleTestList = []
  manyTestList = []
  isoTestList = []
  mpiTestList = []
  testsRun = 0
  testsPassed = 0

  nParticles = 1
  nEvents = 1000
  for pdgid in pdgids:
    singleTestList.append(TestParticle(pdgid,nParticles,nEvents,debug))
  print("Running Single Particle Gun Tests...")
  for test in singleTestList:
    test.run()

  nParticles = 10
  nEvents = 1000
  for pdgid in pdgids:
    manyTestList.append(TestParticle(pdgid,nParticles,nEvents,debug))
  print("Running Many Particle Gun Tests...")
  for test in manyTestList:
    test.run()

  nParticles = 1
  nEvents = 1000
  for pdgid in [11,13,22,5]:
    isoTestList.append(IsoAndBTagTests(pdgid,nParticles,nEvents,True,debug))
    isoTestList.append(IsoAndBTagTests(pdgid,nParticles,nEvents,False,debug))
  print("Running Iso/b-Tag Tests...")
  for test in isoTestList:
    test.run()

  nParticles = 1
  nEvents = 10
  for weight,mpi in zip([None,None,0.5,1.0,50.],[1,11,13,22,5]):
    mpiTestList.append(TestMPIWeight(mpi,weight,nEvents,debug))
  print("Running MPI/Weight Tests...")
  for test in mpiTestList:
    test.run()
  
  nParticles = 1
  nEvents = 100000
  CONFIGFILE="cmsConfig.yaml"
  resolutionTest = ResolutionTests(nParticles,nEvents,debug)
  print("Running Resolution Tests...")
  resolutionTest.run()

  print("")
  print("")
  print("########################################")
  print("### Single Particle Efficiency Tests ###")
  print("########################################")
  print("")

  for test in singleTestList:
    testsRun += 1
    if test.report():
        testsPassed += 1

  print("")
  print("########################################")
  print("### Many Particle Efficiency Tests #####")
  print("########################################")
  print("")
  print("N Particles: %i" % nParticles)
  print("")

  for test in manyTestList:
    testsRun += 1
    if test.report():
        testsPassed += 1

  print("")
  print("########################################")
  print("####### Isolation/b-Tag Tests ##########")
  print("########################################")
  print("")

  for test in isoTestList:
    testsRun += 1
    if test.report():
        testsPassed += 1

  print("")
  print("########################################")
  print("######### MPI/Weight Tests #############")
  print("########################################")
  print("")

  for test in mpiTestList:
    testsRun += 1
    if test.report():
        testsPassed += 1

  print("")
  print("########################################")
  print("########## Resolution Tests ############")
  print("########################################")
  print("")

  tmpPassed, tmpTried = resolutionTest.report()
  testsRun += tmpTried
  testsPassed += tmpPassed

  print("")
  print("********************************")
  print("Tests Passed: %i/%i" % (testsPassed,testsRun))
  print("********************************")
  print("Done.")
  print("")

