// SPODS--Simple Physics Object based Detector Simulation
// Copyright (C) 2011, 2012, 2013 Justin Hugon

#include <TSystem.h>
#include <TROOT.h>
#include <TClonesArray.h>
#include <TParticle.h>
#include <iostream>
#include <TFile.h>
#include <TTree.h>
#include <TH2F.h>
#include <TH1F.h>
#include <TProfile.h>
#include <TCanvas.h>
#include <TStyle.h>
#include <TLegend.h>
#include <TF1.h>

#include "PhysicsObjectFunctions.h"

void analyze(
  TClonesArray* electrons,
  TClonesArray* muons,
  TClonesArray* photons,
  TClonesArray* jets,
  double& met,
  double& metphi

); //TClonesArray of TParticles

void getHighestPtParticle(TClonesArray*,TParticle*);
void fillResHist(TClonesArray* particles, TH2F* resHist, std::string option);
TH1D* getJetResolution(TH2F* ratioVPt);

int main(int argc, char *argv[])
{
  if (argc != 2) 
  {
	std::cout << "testReader requires 1 argument:" << std::endl
	<< "testReader <infilename.root>" << std::endl;
	return 1;
  }

  gStyle->SetOptStat(0);

  TCanvas* c1 = new TCanvas("c1");
  TFile* f = new TFile(argv[1]);
  TTree * tree = (TTree*) f->Get("tree");

  TClonesArray* muons = new TClonesArray("TParticle");
  TClonesArray* electrons = new TClonesArray("TParticle");
  TClonesArray* photons = new TClonesArray("TParticle");
  TClonesArray* jets = new TClonesArray("TParticle");
  double met=0.0;
  double metPhi=0.0;
  double sumPt=0.0;
  double trueMet=0.0;
  double trueMetPhi=0.0;
  double trueSumPt=0.0;

  tree->SetBranchAddress("muons",&muons);
  tree->SetBranchAddress("electrons",&electrons);
  tree->SetBranchAddress("photons",&photons);
  tree->SetBranchAddress("jets",&jets);
  tree->SetBranchAddress("met",&met);
  tree->SetBranchAddress("metPhi",&metPhi);
  tree->SetBranchAddress("sumPt",&sumPt);
  tree->SetBranchAddress("trueMet",&trueMet);
  tree->SetBranchAddress("trueMetPhi",&trueMetPhi);
  tree->SetBranchAddress("trueSumPt",&trueSumPt);

  TH1F* mass = new TH1F("mass","jj Mass",50,100,300);
  TH1F* pt = new TH1F("pt","j Pt",50,50,150);
  pt->Sumw2();
  mass->Sumw2();
  
  unsigned nEvents = tree->GetEntries();
  for(unsigned i=0; i<nEvents;i++)
  {
    tree->GetEvent(i);

    unsigned nMuons = muons->GetEntries();
    unsigned nElectrons = electrons->GetEntries();
    unsigned nPhotons = photons->GetEntries();
    unsigned nJets = jets->GetEntries();

/*
    for(unsigned i=0; i<nJets; i++)
    {
	TParticle* jet = (TParticle*) jets->At(i);
	//bool isB = getGenB(jet);
	//bool isBTag = getBTag(jet);
	mass->Fill(jet->M());
	pt->Fill(jet->Pt());
    }
*/
    if(jets->GetEntries() < 2)
	continue;
    
    TParticle* jet1 = (TParticle*) jets->At(0);
    TParticle* jet2 = (TParticle*) jets->At(1);

    TLorentzVector jetP1;
    TLorentzVector jetP2;
    jet1->Momentum(jetP1);
    jet2->Momentum(jetP2);

    //jetP1 *= getGenScalingFactor(jet1);
    //jetP2 *= getGenScalingFactor(jet1);

    TLorentzVector dijetP = jetP1 +jetP2;

    pt->Fill(jet1->Pt());
    pt->Fill(jet2->Pt());
    mass->Fill(dijetP.M());
  }

/*
  muRelIso->SetLineColor(kRed+1);
  elRelIso->SetLineColor(kGreen+1);
  phRelIso->SetLineColor(kYellow+1);
  TLegend* isoLegend = new TLegend(0.7,0.70,0.9,0.9);
  isoLegend->SetFillColor(kWhite);
  isoLegend->AddEntry(muRelIso,"#mu Iso","l");
  isoLegend->AddEntry(elRelIso,"e Iso","l");
  isoLegend->AddEntry(phRelIso,"#gamma Iso","l");
  isoLegend->Draw("same");
  c1->SaveAs("iso.png");
*/

  TF1* fpt = new TF1("fpt","gaus(0)",80,120);
  fpt->SetParLimits(1,80,110);
  fpt->SetParLimits(2,10,50);
  TF1* fmass = new TF1("fmass","gaus(0)",175,210);
  fmass->SetParLimits(1,160,210);
  fmass->SetParLimits(2,10,50);
  pt->Fit(fpt,"RIL");
  mass->Fit(fmass,"RIL");

  pt->Draw();
  c1->SaveAs("jetPt.png");
  mass->Draw();
  c1->SaveAs("jetMass.png");

  return 0;
}

void getHighestPtParticle(TClonesArray* particles, TParticle* highestPtPart)
{
  unsigned nPart = particles->GetEntries();
  float highestPt = 0.0;

  for(unsigned i=0; i<nPart; i++)
  {
	TParticle* tmpPart = (TParticle*) (particles->At(i));
	if(tmpPart->Pt() > highestPt)
		highestPtPart = tmpPart;
  }
}

void fillResHist(TClonesArray* particles, TH2F* resHist, std::string option)
{
  unsigned nPart = particles->GetEntries();
  if (option == std::string("invert"))
   for(unsigned iPart = 0; iPart<nPart;iPart++)
   {
    TParticle* tmpPart = (TParticle*) (particles->At(iPart));
    resHist->Fill(tmpPart->Pt()/tmpPart->GetCalcMass(),1.0/tmpPart->GetCalcMass());
   }
  else
   for(unsigned iPart = 0; iPart<nPart;iPart++)
   {
    TParticle* tmpPart = (TParticle*) (particles->At(iPart));
    resHist->Fill(tmpPart->Pt()/tmpPart->GetCalcMass(),tmpPart->GetCalcMass());
   }
}

TH1D* getJetResolution(TH2F* ratioVPt)
{
  std::string meanHistName = ratioVPt->GetName();
  std::string sigmaHistName = meanHistName;
  meanHistName.append("_1");
  sigmaHistName.append("_2");

  ratioVPt->FitSlicesY();

  TH1D* outMeanHist = (TH1D*)gDirectory->Get(meanHistName.c_str()); //mean
  TH1D* outSigmaHist = (TH1D*)gDirectory->Get(sigmaHistName.c_str()); //sigma
  
  outSigmaHist->SetTitle(ratioVPt->GetTitle());
  outSigmaHist->GetXaxis()->SetTitle(ratioVPt->GetXaxis()->GetTitle());
  std::string ytitle = "#sigma(";
  ytitle.append(ratioVPt->GetYaxis()->GetTitle());
  ytitle.append(")/<");
  ytitle.append(ratioVPt->GetYaxis()->GetTitle());
  ytitle.append(">");
  outSigmaHist->GetYaxis()->SetTitle(ytitle.c_str());

  outSigmaHist->Divide(outMeanHist);

  return outSigmaHist;
}
