// SPODS--Simple Physics Object based Detector Simulation
// Copyright (C) 2011, 2012, 2013 Justin Hugon

#include <TSystem.h>
#include <TROOT.h>
#include <TClonesArray.h>
#include <TParticle.h>
#include <iostream>
#include <TFile.h>
#include <TTree.h>
#include <TH2F.h>
#include <TH1F.h>
#include <TProfile.h>
#include <TCanvas.h>
#include <TStyle.h>
#include <TLegend.h>
#include <TF1.h>

#include "PhysicsObjectFunctions.h"

void analyze(
  TClonesArray* electrons,
  TClonesArray* muons,
  TClonesArray* photons,
  TClonesArray* jets,
  double& met,
  double& metphi

); //TClonesArray of TParticles

void getHighestPtParticle(TClonesArray*,TParticle*);
void fillResHist(TClonesArray* particles, TH2F* resHist, std::string option);
TH1D* getJetResolution(TH2F* ratioVPt);
double getDiM(TClonesArray*);

int main(int argc, char *argv[])
{
  if (argc != 2) 
  {
	std::cout << "testReader requires 1 argument:" << std::endl
	<< "testReader <infilename.root>" << std::endl;
	return 1;
  }

  gStyle->SetOptStat(0);

  TCanvas* c1 = new TCanvas("c1");
  TFile* f = new TFile(argv[1]);
  TTree * tree = (TTree*) f->Get("tree");
  TH2F* htemp;
  TH1D *htemp_2;

  tree->Draw("1.0/muons->GetCalcMass():muons->Pt()*muons->GetCalcMass()","","colz");
  htemp = (TH2F*) gPad->GetPrimitive("htemp");
  if(htemp)
  {
    htemp->SetTitle("#mu Resolution");
    htemp->GetXaxis()->SetTitle("#mu p_{T}^{Gen}");
    htemp->GetYaxis()->SetTitle("p_{T}^{Smeared}/p_{T}^{Gen}");
    htemp->FitSlicesY();
    htemp_2 = (TH1D*)gDirectory->Get("htemp_2");
    c1->Update();
    c1->Print("muRes2D.png","png");
    c1->Clear();
    htemp_2->SetTitle("#mu Resolution #sigma v. p_{T}");
    htemp_2->GetYaxis()->SetTitle("#sigma(smeared/gen)");
    htemp_2->GetXaxis()->SetRange(1,htemp_2->GetNbinsX()-3);
    htemp_2->Draw();
    c1->Print("muRes.png","png");
    c1->Clear();
  }

  tree->Draw("1.0/electrons->GetCalcMass():electrons->Pt()*electrons->GetCalcMass()","","colz");
  htemp = (TH2F*) gPad->GetPrimitive("htemp");
  if(htemp)
  {
    htemp->SetTitle("e Resolution");
    htemp->GetXaxis()->SetTitle("e p_{T}^{Gen}");
    htemp->GetYaxis()->SetTitle("p_{T}^{Smeared}/p_{T}^{Gen}");
    htemp->FitSlicesY();
    htemp_2 = (TH1D*)gDirectory->Get("htemp_2");
    c1->Update();
    c1->Print("elRes2D.png","png");
    c1->Clear();
    htemp_2->SetTitle("e Resolution #sigma v. p_{T}");
    htemp_2->GetYaxis()->SetTitle("#sigma(smeared/gen)");
    htemp_2->GetXaxis()->SetRange(1,htemp_2->GetNbinsX()-3);
    htemp_2->Draw();
    c1->Print("elRes.png","png");
    c1->Clear();
  }

  tree->Draw("1.0/photons->GetCalcMass():photons->Pt()*photons->GetCalcMass()","","colz");
  htemp = (TH2F*) gPad->GetPrimitive("htemp");
  if(htemp)
  {
    htemp->SetTitle("#gamma Resolution");
    htemp->GetXaxis()->SetTitle("#gamma E_{T}^{Gen}");
    htemp->GetYaxis()->SetTitle("E_{T}^{Smeared}/E_{T}^{Gen}");
    htemp->FitSlicesY();
    htemp_2 = (TH1D*)gDirectory->Get("htemp_2");
    c1->Update();
    c1->Print("phRes2D.png","png");
    c1->Clear();
    htemp_2->SetTitle("#gamma Resolution #sigma v. E_{T}");
    htemp_2->GetYaxis()->SetTitle("#sigma(smeared/gen)");
    htemp_2->GetXaxis()->SetRange(1,htemp_2->GetNbinsX()-3);
    htemp_2->Draw();
    c1->Print("phRes.png","png");
    c1->Clear();
  }

  tree->Draw("1.0/jets->GetCalcMass():jets->Pt()*jets->GetCalcMass()","","colz");
  htemp = (TH2F*) gPad->GetPrimitive("htemp");
  if(htemp)
  {
    htemp->SetTitle("Jet Resolution");
    htemp->GetXaxis()->SetTitle("Jet E_{T}^{Gen}");
    htemp->GetYaxis()->SetTitle("E_{T}^{Smeared}/E_{T}^{Gen}");
    c1->Update();
    c1->Print("jetRes2D.png","png");

    TH1D* jetRes = getJetResolution(htemp);
    jetRes->GetXaxis()->SetRange(1,jetRes->GetNbinsX()-3);
    jetRes->Draw();
    c1->Update();
    c1->Print("jetRes.png","png");
    c1->Clear();
  }

  tree->Draw("met");
  TH1F* htemp1f = (TH1F*) gPad->GetPrimitive("htemp");
  htemp1f->SetTitle("E_{T}^{Miss}");
  htemp1f->GetXaxis()->SetTitle("E_{T}^{Miss} [GeV]");
  htemp1f->GetYaxis()->SetTitle("Events");
  htemp1f->SetLineColor(kRed+1);
  htemp1f->Draw();
  tree->Draw("trueMet","","same");
  c1->SaveAs("met.png");
  tree->Draw("sumPt","");
  htemp1f = (TH1F*) gPad->GetPrimitive("htemp");
  htemp1f->SetTitle("#sum P_{T} [GeV]");
  htemp1f->GetXaxis()->SetTitle("#sum P_{T} [GeV]");
  htemp1f->GetYaxis()->SetTitle("Events");
  htemp1f->SetLineColor(kRed+1);
  htemp1f->Draw();
  tree->Draw("trueSumPt","","same");
  c1->SaveAs("sumPt.png");


  tree->Draw("met*cos(metPhi):sumPt","","colz");
  htemp = (TH2F*) gPad->GetPrimitive("htemp");
  if(htemp)
  {
    TF1* metFitFunc = new TF1("metFitFunc","sqrt([0]*[0]+[1]*[1]*(x-[3])+[2]*[2]*pow(x-[3],2))",0.0,1400.0);
    htemp->SetTitle("MET #sigma_{x}");
    htemp->GetYaxis()->SetTitle("E_{T}^{Miss} x-Component [GeV]");
    htemp->GetXaxis()->SetTitle("Sum E_{T} [GeV]");
    htemp->FitSlicesY();
    htemp_2 = (TH1D*)gDirectory->Get("htemp_2");
    c1->Update();
    c1->Print("metRes2D.png","png");
    c1->Clear();
    metFitFunc->SetParameters(5.4,1.23,0.019,250.0);
    metFitFunc->SetParLimits(0,0.001,50);
    metFitFunc->SetParLimits(1,0.1,20);
    //metFitFunc->SetParLimits(2,0.000,0.000);
    metFitFunc->SetParLimits(3,100.0,300.0);
    metFitFunc->FixParameter(2,0.000);
    htemp_2->Fit(metFitFunc,"I");
    htemp_2->SetTitle("#sigma_{x} E_{T}^{Miss} v. Sum E_{T}");
    htemp_2->GetYaxis()->SetTitle("#sigma_{x} E_{T}^{Miss} [GeV]");
    //htemp_2->GetXaxis()->SetRange(1,htemp_2->GetNbinsX()-15);
    //htemp_2->GetXaxis()->SetRangeUser(0.0,250.0);
    htemp_2->GetYaxis()->SetRangeUser(0.0,200.0);
    htemp_2->Draw();
    c1->Print("metRes.png","png");
    c1->Clear();
  }

  TClonesArray* muons = new TClonesArray("TParticle");
  TClonesArray* electrons = new TClonesArray("TParticle");
  TClonesArray* photons = new TClonesArray("TParticle");
  TClonesArray* jets = new TClonesArray("TParticle");
  double met=0.0;
  double metPhi=0.0;
  double sumPt=0.0;
  double trueMet=0.0;
  double trueMetPhi=0.0;
  double trueSumPt=0.0;

  tree->SetBranchAddress("muons",&muons);
  tree->SetBranchAddress("electrons",&electrons);
  tree->SetBranchAddress("photons",&photons);
  tree->SetBranchAddress("jets",&jets);
  tree->SetBranchAddress("met",&met);
  tree->SetBranchAddress("metPhi",&metPhi);
  tree->SetBranchAddress("sumPt",&sumPt);
  tree->SetBranchAddress("trueMet",&trueMet);
  tree->SetBranchAddress("trueMetPhi",&trueMetPhi);
  tree->SetBranchAddress("trueSumPt",&trueSumPt);

  TH1F* muRelIso = new TH1F("muRelIso","Relative Isolation",20,0,2);
  TH1F* elRelIso = new TH1F("elRelIso","Relative Isolation",20,0,2);
  TH1F* phRelIso = new TH1F("phRelIso","Relative Isolation",20,0,2);

  TH1F* mDiMu = new TH1F("mDiMu","DiMuon Mass",50,0,200);
  TH1F* mDiEl = new TH1F("mDiEl","DiElectron Mass",50,0,200);
  TH1F* mDiJet = new TH1F("mDiJet","DiJet Mass",50,0,200);
  TH1F* mDiPh = new TH1F("mDiPh","DiPhoton Mass",50,0,200);
  
  unsigned nLightJets = 0;
  unsigned nBJets = 0;
  unsigned nBTagLightJets = 0;
  unsigned nBTagBJets = 0;
  unsigned nEvents = tree->GetEntries();
  for(unsigned i=0; i<nEvents;i++)
  {
    tree->GetEvent(i);

    unsigned nMuons = muons->GetEntries();
    unsigned nElectrons = electrons->GetEntries();
    unsigned nPhotons = photons->GetEntries();
    unsigned nJets = jets->GetEntries();

    for(unsigned i=0; i<nMuons; i++)
    {
	TParticle* tmpMuon = (TParticle*) muons->At(i);
	muRelIso->Fill(getIso(tmpMuon)/tmpMuon->Pt());
    }
    for(unsigned i=0; i<nElectrons; i++)
    {
	TParticle* tmpEl = (TParticle*) electrons->At(i);
	elRelIso->Fill(getIso(tmpEl)/tmpEl->Pt());
    }
    for(unsigned i=0; i<nPhotons; i++)
    {
	TParticle* tmpPh = (TParticle*) photons->At(i);
	phRelIso->Fill(getIso(tmpPh)/tmpPh->Pt());
    }
    for(unsigned i=0; i<nJets; i++)
    {
	bool isB = getGenB((TParticle*) jets->At(i));
	bool isBTag = getBTag((TParticle*) jets->At(i));
	if(isB)
	{
	  nBJets++;
	  if(isBTag)
	    nBTagBJets++;
	}
	else
	{
	  nLightJets++;
	  if(isBTag)
	    nBTagLightJets++;
	}
    }
	

    if(nMuons>1)
        mDiMu->Fill(getDiM(muons));
    if(nElectrons>1)
        mDiEl->Fill(getDiM(electrons));
    if(nPhotons>1)
        mDiPh->Fill(getDiM(photons));
    if(nJets>1)
        mDiJet->Fill(getDiM(jets));
/*
    getBTag();
    getGenB();
    getCaloIso();
    getTrkIso();
*/
  }

  muRelIso->SetLineColor(kRed+1);
  elRelIso->SetLineColor(kGreen+1);
  phRelIso->SetLineColor(kYellow+1);
  
  muRelIso->GetXaxis()->SetTitle("Relative Isolation");
  muRelIso->GetYaxis()->SetTitle("Counts");
  muRelIso->Draw();
  elRelIso->Draw("same");
  phRelIso->Draw("same");
  TLegend* isoLegend = new TLegend(0.7,0.70,0.9,0.9);
  isoLegend->SetFillColor(kWhite);
  isoLegend->AddEntry(muRelIso,"#mu Iso","l");
  isoLegend->AddEntry(elRelIso,"e Iso","l");
  isoLegend->AddEntry(phRelIso,"#gamma Iso","l");
  isoLegend->Draw("same");
  c1->SaveAs("iso.png");

  std::cout << "\n##########################\nb-Tags: \n";
  std::cout << "Mistag Rate: " << float(nBTagLightJets)/float(nLightJets)<< std::endl;
  std::cout << "BTag Eff:    " << float(nBTagBJets)/float(nBJets)<< 
							std::endl << std::endl;

  c1->Clear();
  mDiMu->Draw();
  c1->SaveAs("mDiMu.png");
  c1->Clear();
  mDiEl->Draw();
  c1->SaveAs("mDiEl.png");
  c1->Clear();
  mDiJet->Draw();
  c1->SaveAs("mDiJet.png");
  c1->Clear();
  mDiPh->Draw();
  c1->SaveAs("mDiPh.png");

  return 0;
}

void getHighestPtParticle(TClonesArray* particles, TParticle* highestPtPart)
{
  unsigned nPart = particles->GetEntries();
  float highestPt = 0.0;

  for(unsigned i=0; i<nPart; i++)
  {
	TParticle* tmpPart = (TParticle*) (particles->At(i));
	if(tmpPart->Pt() > highestPt)
		highestPtPart = tmpPart;
  }
}

void fillResHist(TClonesArray* particles, TH2F* resHist, std::string option)
{
  unsigned nPart = particles->GetEntries();
  if (option == std::string("invert"))
   for(unsigned iPart = 0; iPart<nPart;iPart++)
   {
    TParticle* tmpPart = (TParticle*) (particles->At(iPart));
    resHist->Fill(tmpPart->Pt()/tmpPart->GetCalcMass(),1.0/tmpPart->GetCalcMass());
   }
  else
   for(unsigned iPart = 0; iPart<nPart;iPart++)
   {
    TParticle* tmpPart = (TParticle*) (particles->At(iPart));
    resHist->Fill(tmpPart->Pt()/tmpPart->GetCalcMass(),tmpPart->GetCalcMass());
   }
}

TH1D* getJetResolution(TH2F* ratioVPt)
{
  std::string meanHistName = ratioVPt->GetName();
  std::string sigmaHistName = meanHistName;
  meanHistName.append("_1");
  sigmaHistName.append("_2");

  ratioVPt->FitSlicesY();

  TH1D* outMeanHist = (TH1D*)gDirectory->Get(meanHistName.c_str()); //mean
  TH1D* outSigmaHist = (TH1D*)gDirectory->Get(sigmaHistName.c_str()); //sigma
  
  outSigmaHist->SetTitle(ratioVPt->GetTitle());
  outSigmaHist->GetXaxis()->SetTitle(ratioVPt->GetXaxis()->GetTitle());
  std::string ytitle = "#sigma(";
  ytitle.append(ratioVPt->GetYaxis()->GetTitle());
  ytitle.append(")/<");
  ytitle.append(ratioVPt->GetYaxis()->GetTitle());
  ytitle.append(">");
  outSigmaHist->GetYaxis()->SetTitle(ytitle.c_str());

  outSigmaHist->Divide(outMeanHist);

  return outSigmaHist;
}

double getDiM(TClonesArray* particles)
{
        if(particles->GetEntries() < 2)
            return -1.0;
        TParticle* part0 = (TParticle*) particles->At(0);
        TParticle* part1 = (TParticle*) particles->At(1);
        TLorentzVector p0;
        TLorentzVector p1;
        part0->Momentum(p0);
        part1->Momentum(p1);
        return (p0+p1).M();
}
